/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
//#include "analyzer.h"

using namespace std;
using namespace Moses1;
using namespace Pythia8;
//using namespace RooFit;

int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/";
	string outFileName = dirName + "Kinematics";
	cout << "\n" << endl;

	bool isSmeared = false;
	int nIsSmeared = 0;
	cout << "select true(1) or smeared(2)...";
	cin >> nIsSmeared;
	isSmeared = (nIsSmeared==2) ? true : false;
	
	bool plotOnlyTrue = false;
	int nPlotOnlyTrue = 0;
	cout << "select plot true(1) or plot smeared(2)...";
        cin >> nPlotOnlyTrue;
        plotOnlyTrue = (nPlotOnlyTrue==1) ? true : false;

	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	bool doTrue = true;
        anlz.setTrue(doTrue);
	double average=0.;
	double sigma=1.;
	double resolution=0.12;   // 12%
	double pTTrackScale=1.e3; // 1 TeV
	bool   printChanges = false;
	if(isSmeared) {
		anlz.setSmear(average, sigma, resolution, pTTrackScale, printChanges);
	}
	
	Particle particle;


	Int_t invMnBins = 50;
        Double_t invMmin = 1000.,    invMmax = 6000.;
       
	Int_t cosThetanBins = 22;
	Double_t cosThetamin = -1., cosThetamax = 1.;
	
	Int_t    pTnBins = 50; // 100
        Double_t pTmin   = 10.;
        Double_t pTmax   = 4000.;
        Double_t pTBoundToReduce = 1000.;

        Int_t    etanBins = 48; // 80
        Double_t etamin   = -3.;
        Double_t etamax   = 3.;

	Int_t    xBnBins = 50; //100
        Double_t xBmin   = 0.;  
        Double_t xBmax   = 1.;


	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=10.;
	double EtaCut=2.5;
	int cutSelector = 1;
	cout << "select cuts to apply: no cuts (-1) all cuts (0), only mHat cut (1), only pT cut (2), only eta cut (3)..." << endl;
	cin >> cutSelector;
	switch(cutSelector) {
		case -1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 invMassCut = 0.;
			 break;
		case  0: break;
		case  1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  2: invMassCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  3: invMassCut = 0.;
			 pTransverseCut = 0.;
			 break;
		default: cout << "wrong option cutSelector = " << cutSelector << "\t set to default cutSelector=0" << endl;
			 break;
	}
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);


/*
	//--- variables to put in the trees for RooFit
	Int_t    eventNumber;
	Double_t costhetaCS;
	Double_t costhetaCSmax;

	// declare the .root file for the trees for RooFit
	string cosThetaRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+"_RooFit.root");
	TFile *roo_treeFile = new TFile(cosThetaRootFileName.c_str(),"recreate");

	//--- trees to put cos(theta) in
	TTree *tree_SM = new TTree("tree_SM","tree_SM");
	tree_SM->Branch("roo_eventNumber",&eventNumber,"eventNumber/I");
	tree_SM->Branch("roo_cosTheta",&costhetaCS,"cosThetaCS/D");
	tree_SM->Branch("roo_cosThetaMax",&costhetaCSmax,"cosThetaMax/D");

	TTree *tree_E6 = new TTree("tree_E6","tree_E6");
	tree_E6->Branch("roo_eventNumber",&eventNumber,"eventNumber/I");
	tree_E6->Branch("roo_cosTheta",&costhetaCS,"cosThetaCS/D");
	tree_E6->Branch("roo_cosThetaMax",&costhetaCSmax,"cosThetaMax/D");

	TTree *tree_KK = new TTree("tree_KK","tree_KK");
	tree_KK->Branch("roo_eventNumber",&eventNumber,"eventNumber/I");
	tree_KK->Branch("roo_cosTheta",&costhetaCS,"cosThetaCS/D");
	tree_KK->Branch("roo_cosThetaMax",&costhetaCSmax,"cosThetaMax/D");
*/


	// choose reference frame
	int referenceFrameSelector = 0;
	cout << "choose reference frame to calculate cos(theta)" << endl;
	cout << "(1) Helicity frame or (2) Collins-Soper frame" << endl;
	cout << "..." << endl;
	cin >> referenceFrameSelector;
	if (referenceFrameSelector != 1 && referenceFrameSelector != 2) {
		cout << "invalid option, setting to default = 1" << endl;
		referenceFrameSelector = 1;
	}
	string cosThetaTitle;
	if (referenceFrameSelector == 1) {cosThetaTitle = "Helicity";}
	else if (referenceFrameSelector == 2) {cosThetaTitle = "Collins-Soper";}

	
	int sampleSizeSelector = 1;
	cout << "select input sample size: 100k(1), L=500fb^-1(2), L=100fb^-1(3)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector<1 || sampleSizeSelector>3) {
		cout << "invalid option, set to 1 (default)" << endl;
		sampleSizeSelector = 1;
	}
	string sSampleSize = "";
	switch (sampleSizeSelector) {
		case 1: sSampleSize = "_LrefinvFb"; break;
		case 2: sSampleSize = "_L500invFb"; break;
		case 3: sSampleSize = "_L100invFb"; break;
		default: break;
	}

	int refModelSelector = 1;
	cout << "select reference sample: KK(1), SM(2), E6(3)..." << endl;
	cin >> refModelSelector;
	if (refModelSelector<1 || refModelSelector>3) {
		cout << "invalid option, set to 1 (default)" << endl;
		refModelSelector = 1;
	}
	string sRefSample = "";
	switch (refModelSelector) {
		case 1: sRefSample = "_KKref"; break;
		case 2: sRefSample = "_SMref"; break;
		case 3: sRefSample = "_E6ref"; break;
		default: break;
	}

	outFileName += sSampleSize;
	if(isSmeared && plotOnlyTrue) outFileName += "_true";
	else if (isSmeared && !plotOnlyTrue) outFileName += "_smeared";


 	string s2DPlot = "";
        int n2DPlot = 0;
        cout << "choose 2d plot type: SCAT(0) SURF(1), COLZ(2), CONTZ(3) or BOX(4)... " << endl;
        cin >> n2DPlot;
        if (n2DPlot==0) {
                s2DPlot = "SCAT";
        }
        else if (n2DPlot==1) {
                s2DPlot = "SURF";
        }
        else if (n2DPlot==2) {
                s2DPlot = "COLZ";
        }
        else if (n2DPlot==3) {
                s2DPlot = "CONTZ";
        }
	else if (n2DPlot==4) {
                s2DPlot = "BOX";
        }
        else {
                cout << "invalid option, setting default to CONTZ" << endl;
                s2DPlot = "CONTZ";
        }


	
	string sISRFSRswitch = "";
        int nISRFSRswitch = 0;
        cout << "default(0), +ISR+FSR(1), +ISR-FSR(2), -ISR-FSR(3)" << endl;
        cin >> nISRFSRswitch;
        if (nISRFSRswitch == 0) {
                sISRFSRswitch = "_withISRFSR";
                if(sampleSizeSelector==2 || sampleSizeSelector==3) {
                        pTmin = 1.;
                        pTmax = 4000.;
                        pTBoundToReduce = 2500;
                }
        }
        else if (nISRFSRswitch == 1) {
                sISRFSRswitch = "_withISRFSR";
                if(sampleSizeSelector==2 || sampleSizeSelector==3) {
                        pTmin = 1.;
                        pTmax = 1500.;
                        pTBoundToReduce = 0;
                }
        }
        else if (nISRFSRswitch == 2) {
                sISRFSRswitch = "_noFSR";
                pTmin = 1.;
                pTmax = 1500.;
                pTBoundToReduce = 0;
        }
        else if (nISRFSRswitch == 3) {
                sISRFSRswitch = "_noISRnoFSR";
                pTmin = 1.;
                pTmax = 20.;
                pTBoundToReduce = 0;
        }
        else {
                cout << "invalid option, setting to default: withISRFSR" << endl;
                sISRFSRswitch = "_withISRFSR";
        }




	double cosThMax = 1.;
        cout << "enter a limit for |cos(theta)|...." << endl;
        cin >> cosThMax;
        if (cosThMax > 1. || cosThMax < 0.) {
                cout << "invalid choice: |cos(theta)|=" << cosThMax << "  setting it to |cos(theta)|=1" << endl;
                cosThMax = 1.;
        }

        string treeType = "testTree";
        int treeTypeSelector = 1;
        cout << "select tree type: testTree(1), testTree_allPairs(2)..." << endl;
        cin >> treeTypeSelector;
        switch(treeTypeSelector) {
                case 1: treeType = "testTree"; break;
                case 2: treeType = "testTree_allPairs"; break;
                default: cout << "wrong option: treeTypeSelector=" << treeTypeSelector << "\t setting it to default: treeTypeSelector=1" << endl;
                        treeType = "testTree";
                        break;
        }




	// declare kinematics histograms and root items
	string kinematicsRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + sRefSample + ".root");
	string kinematicsEpsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + sRefSample + ".eps");
        string bjorkenEpsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_bjorken.eps");
        string QTepsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+"_QT.eps");


	TFile *kinematicsHistFile = new TFile(kinematicsRootFileName.c_str(),"recreate");


	TDirectory* dir_allHistograms = kinematicsHistFile->mkdir("allHistograms");
        dir_allHistograms->cd();


        TDirectory* dir_kinematics = kinematicsHistFile->mkdir("kinematics");
        dir_kinematics->cd();
        TCanvas *canv_kinematics = new TCanvas("Kinematics" ,"Kinematics",0,0,1400,800);
        canv_kinematics->Divide(2,2);
	TVirtualPad *pad_mHat=canv_kinematics->cd(1);
	pad_mHat->SetLogy();
	TVirtualPad *pad_pT=canv_kinematics->cd(3);
	pad_pT->SetLogy();

	
	TDirectory* dir_mHat = kinematicsHistFile->mkdir("mHat");
        dir_mHat->cd();
	TCanvas *canv_mHat = new TCanvas("mHat" ,"mHat",0,0,1400,800);
	canv_mHat->SetLogy();
	TDirectory* dir_pT = kinematicsHistFile->mkdir("pT");
        dir_pT->cd();
	TCanvas *canv_pT = new TCanvas("pT" ,"pT",0,0,1400,800);
        canv_pT->SetLogy();
	TDirectory* dir_eta = kinematicsHistFile->mkdir("eta");
        dir_eta->cd();
	TCanvas *canv_eta = new TCanvas("eta" ,"eta",0,0,1400,800);
	TDirectory* dir_cosTheta = kinematicsHistFile->mkdir("cosTheta");
        dir_cosTheta->cd();
	TCanvas *canv_cosTheta = new TCanvas("cosTheta" ,"cosTheta",0,0,1400,800);

	
	//TFile *bjorkenHistFile = new TFile(bjorkenRootFileName.c_str(),"recreate");
	TDirectory* dir_bjorken = kinematicsHistFile->mkdir("Bjorken");
        dir_bjorken->cd();
	TCanvas *canv_bjorken = new TCanvas("canv_bjorken","Bjorken",0,0,1400,800);
	canv_bjorken->Divide(1,2);


	//TFile *QThistFile = new TFile(QTrootFileName.c_str(),"recreate");
	TDirectory* dir_QT = kinematicsHistFile->mkdir("QT");
        dir_QT->cd();
	TCanvas *canv_QT = new TCanvas("canv_QT","QT",0,0,1400,800);
	canv_QT->SetLogy();
	TDirectory* dir_QTvsMhat = dir_QT->mkdir("QTvsMhat");
        dir_QTvsMhat->cd();
	TCanvas *canv_QTvsMhat = new TCanvas("canv_QTvsMhat","QTvsMhat",0,0,1400,800);



	dir_allHistograms->cd();

	// binnings - corresponding to the best Kolmogorov test
	// 100[fb^-1] cosTheta
	//    -- SM ref 22 bins v
	//    -- E6 ref 16 bins v
	//    -- KK ref 22 bins v
	// 100[fb^-1] mHat
	//    -- SM ref 20 bins v
	//    -- E6 ref 12 bins v
	//    -- KK ref 20 bins v
	// 500[fb^-1] cosTheta
	//    -- SM ref 20 bins v
	//    -- E6 ref 20 bins v
	//    -- KK ref 20 bins v
	// 500[fb^-1] mHat
	//    -- SM ref 14 bins v
	//    -- E6 ref 20 bins v
	//    -- KK ref 20 bins v

	// ------------- new binnings:
	// 100[fb^-1] cosTheta
        //    -- SM ref 16 bins v
        //    -- E6 ref 8  bins v
        //    -- KK ref 8  bins v
        // 100[fb^-1] mHat
        //    -- SM ref 10 bins v
        //    -- E6 ref 12 bins v
        //    -- KK ref 10 bins v
        // 500[fb^-1] cosTheta
        //    -- SM ref 10 bins v
        //    -- E6 ref 10 bins v
        //    -- KK ref 10 bins v
        // 500[fb^-1] mHat
        //    -- SM ref 10 bins v
        //    -- E6 ref 10 bins v
        //    -- KK ref 10 bins v


	// choose mHat & cosTheta binnings
	switch(sampleSizeSelector) {
		case 1:	invMnBins = 50; cosThetanBins = 22; break; // 1M sample
		case 2: // 500[fb^-1] sample
			switch(refModelSelector) {
				//case 1: invMnBins = 20; cosThetanBins = 20; break; // KK ref
				//case 2: invMnBins = 14; cosThetanBins = 20; break; // SM ref
				//case 3: invMnBins = 20; cosThetanBins = 20; break; // E6 ref
				case 1: invMnBins = 10; cosThetanBins = 10; break; // KK ref
				case 2: invMnBins = 10; cosThetanBins = 10; break; // SM ref
				case 3: invMnBins = 10; cosThetanBins = 10; break; // E6 ref
				default: break;
			}
			invMnBins += 4; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			break;
		case 3: // 100[fb^-1] sample
			switch(refModelSelector) {
				//case 1: invMnBins = 20; cosThetanBins = 22; break; // KK ref
				//case 2: invMnBins = 20; cosThetanBins = 22; break; // SM ref
				//case 3: invMnBins = 12; cosThetanBins = 16; break; // E6 ref
				case 1: invMnBins = 10; cosThetanBins = 8; break; // KK ref
				case 2: invMnBins = 10; cosThetanBins = 16; break; // SM ref
				case 3: invMnBins = 12; cosThetanBins = 8; break; // E6 ref
				default: break;
			}
			invMnBins += 4; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			break;
		default: break;
	}


	if (sampleSizeSelector==2 || sampleSizeSelector==3) { // small samples
		pTnBins = 20;
	}

	if (sampleSizeSelector==2 || sampleSizeSelector==3) { // small samples
		etanBins = 12;
	}

/*	
	if (sampleSizeSelector==2) { // small sample
		cosThetanBins = 12;
		cosThetamin   = -1.1;
		cosThetamax   = 1.1;
	}
*/

	if (sampleSizeSelector==2 || sampleSizeSelector==3) { // small samples
		xBnBins = 20;
	}

	cout << "invMnBins     = " << invMnBins << endl;
	cout << "cosThetanBins = " << cosThetanBins << endl;
	cout << "pTnBins       = " << pTnBins << endl;
	cout << "etanBins      = " << etanBins << endl;
	cout << "xBnBins       = " << xBnBins << endl;

	if (cutSelector==-1 || cutSelector==1 || cutSelector==2 || cutSelector==3) {
		pTnBins = 100;
		pTmin = 5.;
		pTmax = 100.;
		
		etanBins = 100;
		etamin = -6.;
		etamax = +6.;

		invMnBins = 500;
		invMmin = 5.;
		invMmax = 6000.;

		cosThetanBins = 120;
		cosThetamin   = -1.1;
		cosThetamax   = 1.1;
	}




	// logarithmic boundries and bins of histograms
	Double_t logMmin = log10(invMmin);
	Double_t logMmax = log10(invMmax - 1000);
	Double_t M_binwidth = (Double_t)( (logMmax-logMmin)/(Double_t)invMnBins );
	Double_t M_bins[invMnBins+1];
	M_bins[0] = invMmin;
	for (Int_t i=1 ; i<=invMnBins ; i++) { M_bins[i] = TMath::Power( 10,(logMmin + i*M_binwidth) ); }

	Double_t logpTmin = log10(pTmin);
	Double_t logpTmax = log10(pTmax - pTBoundToReduce);
	Double_t pT_binwidth = (Double_t)( (logpTmax-logpTmin)/(Double_t)pTnBins );
	Double_t pT_bins[pTnBins+1];
	pT_bins[0] = pTmin;
	for (Int_t i=1 ; i<=pTnBins ; i++) { pT_bins[i] = TMath::Power( 10,(logpTmin + i*pT_binwidth) ); }


	
	// titles for the cos(theta) plots
//	string title  = "SM vs KK vs Z': cosTheta (in " + cosThetaTitle + " frame) *Normalized*";
	string title  = "";
	string xtitle = "cos#theta'";


	// declaration of histograms
	//----- SM -----------------------------------------------------------------------------------------------------
	dir_mHat->cd();
	TH1 *SM_invMass = new TH1D("SM_invMass", "", invMnBins, invMmin, invMmax);
	SM_invMass->SetLineStyle(5);
	SM_invMass->SetLineColor(kBlue);
	SM_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	SM_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *SM_pT = new TH1D("SM_pT", "", pTnBins, pTmin, pTmax);
	SM_pT->SetLineStyle(5);
	SM_pT->SetLineColor(kBlue);
	SM_pT->SetXTitle("p_{T}(#mu^{-})[GeV]");
	SM_pT->SetYTitle("Events");
	
	dir_QT->cd();
	TH1 *SM_QT = new TH1D("SM_QT", "", pTnBins, pTmin, pTmax+500);
	SM_QT->SetLineStyle(5);
	SM_QT->SetLineColor(kBlue);
        SM_QT->SetXTitle("Q_{T}[GeV]");
        SM_QT->SetYTitle("Events");

	dir_eta->cd();	
	TH1 *SM_eta = new TH1D("SM_eta", "", etanBins, etamin, etamax);
	//SM_eta->SetLineStyle(5);
	SM_eta->SetLineColor(kBlue);
	SM_eta->SetMarkerStyle(24);
	SM_eta->SetMarkerColor(kBlue);
	SM_eta->SetXTitle("#eta(#mu^{-})");
	SM_eta->SetYTitle("#frac{1}{#sigma}#times#frac{d#sigma}{d#eta(#mu^{-})}");

	dir_cosTheta->cd();
	TH1 *SM_cosTheta = new TH1D("SM_cosTheta", "", cosThetanBins, cosThetamin, cosThetamax);
	//SM_cosTheta->SetLineStyle(5);
	SM_cosTheta->SetLineColor(kBlue);
	SM_cosTheta->SetMarkerStyle(24);
	SM_cosTheta->SetMarkerColor(kBlue);
	SM_cosTheta->SetXTitle(xtitle.c_str());
	SM_cosTheta->SetYTitle("#frac{1}{#sigma}#times#frac{d#sigma}{dcos#theta'}");
	
	dir_bjorken->cd();
	TH1 *SM_xBjorken_q = new TH1D("SM_xBjorken_q", "", xBnBins, xBmin, xBmax);
	SM_xBjorken_q->SetLineStyle(5);
	SM_xBjorken_q->SetXTitle("x_{q}");
	SM_xBjorken_q->SetYTitle("Events");

	TH1 *SM_xBjorken_qbar = new TH1D("SM_xBjorken_qbar", "", xBnBins, xBmin, xBmax);
	SM_xBjorken_qbar->SetLineStyle(5);
	SM_xBjorken_qbar->SetXTitle("x_{#bar q}");
	SM_xBjorken_qbar->SetYTitle("Events");




	//----- KK -----------------------------------------------------------------------------------------------------
	dir_QTvsMhat->cd();
	TH2 *KK2d_QT = new TH2D("KK2d_QT", "", invMnBins, M_bins, pTnBins, pT_bins);
	KK2d_QT->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	KK2d_QT->SetYTitle("Q_{T}[GeV]");
	KK2d_QT->SetZTitle("Events");

	dir_mHat->cd();
	TH1 *KK_invMass = new TH1D("KK_invMass", "", invMnBins, invMmin, invMmax);
	KK_invMass->SetLineStyle(1);
	KK_invMass->SetLineColor(kRed);
	KK_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	KK_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *KK_pT = new TH1D("KK_pT", "", pTnBins, pTmin, pTmax);
	KK_pT->SetLineStyle(1);
	KK_pT->SetLineColor(kRed);
	KK_pT->SetXTitle("#p_{T}(#mu^{-})[GeV]");
	KK_pT->SetYTitle("Events");
	
	dir_QT->cd();
	TH1 *KK_QT = new TH1D("KK_QT", "", pTnBins, pTmin, pTmax+500);
	KK_QT->SetLineStyle(1);
	KK_QT->SetLineColor(kRed);
	KK_QT->SetXTitle("Q_{T}[GeV]");
	KK_QT->SetYTitle("Events");

	dir_eta->cd();
	TH1 *KK_eta = new TH1D("KK_eta", "", etanBins, etamin, etamax);
	//KK_eta->SetLineStyle(1);
	KK_eta->SetMarkerStyle(21);
	KK_eta->SetMarkerColor(kRed);
	KK_eta->SetLineColor(kRed);
	KK_eta->SetXTitle("#eta(#mu^{-})");
	KK_eta->SetYTitle("#frac{1}{#sigma}#times#frac{d#sigma}{d#eta(#mu^{-})}");

	dir_cosTheta->cd();
	TH1 *KK_cosTheta = new TH1D("KK_cosTheta", "", cosThetanBins, cosThetamin, cosThetamax);
	//KK_cosTheta->SetLineStyle(1);
	KK_cosTheta->SetLineColor(kRed);
	KK_cosTheta->SetMarkerStyle(21);
	KK_cosTheta->SetMarkerColor(kRed);
	KK_cosTheta->SetXTitle(xtitle.c_str());
	KK_cosTheta->SetYTitle("#frac{1}{#sigma}#times#frac{d#sigma}{dcos#theta'}");

	dir_bjorken->cd();
	TH1 *KK_xBjorken_q = new TH1D("KK_xBjorken_q", "", xBnBins, xBmin, xBmax);
	KK_xBjorken_q->SetLineStyle(1);
	KK_xBjorken_q->SetXTitle("x_{q}");
	KK_xBjorken_q->SetYTitle("Events");
	
	TH1 *KK_xBjorken_qbar = new TH1D("KK_xBjorken_qbar", "", xBnBins, xBmin, xBmax);
	KK_xBjorken_qbar->SetLineStyle(1);
	KK_xBjorken_qbar->SetXTitle("x_{#bar q}");
	KK_xBjorken_qbar->SetYTitle("Events");

	
	//----- Z' -----------------------------------------------------------------------------------------------------
	dir_mHat->cd();
	TH1 *Zprime_invMass = new TH1D("Zprime_invMass", "", invMnBins, invMmin, invMmax);
	Zprime_invMass->SetLineStyle(3);
	Zprime_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	Zprime_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *Zprime_pT = new TH1D("Zprime_pT", "", pTnBins, pTmin, pTmax);
	Zprime_pT->SetLineStyle(3);
	Zprime_pT->SetXTitle("#p_{T}(#mu^{-})[GeV]");
	Zprime_pT->SetYTitle("Events");
	
	dir_QT->cd();
	TH1 *Zprime_QT = new TH1D("Zprime_QT", "", pTnBins, pTmin, pTmax+500);
	Zprime_pT->SetLineStyle(3);
	Zprime_QT->SetXTitle("Q_{T}[GeV]");
	Zprime_QT->SetYTitle("Events");

	dir_eta->cd();
	TH1 *Zprime_eta = new TH1D("Zprime_eta", "", etanBins, etamin, etamax);
	//Zprime_eta->SetLineStyle(3);
	Zprime_eta->SetMarkerStyle(3);
	Zprime_eta->SetXTitle("#eta(#mu^{-})");
	Zprime_eta->SetYTitle("#frac{1}{#sigma}#times#frac{d#sigma}{d#eta(#mu^{-})}");

	dir_cosTheta->cd();
	TH1 *Zprime_cosTheta = new TH1D("Zprime_cosTheta", "", cosThetanBins, cosThetamin, cosThetamax);
	//Zprime_cosTheta->SetLineStyle(3);
	Zprime_cosTheta->SetMarkerStyle(3);
	Zprime_cosTheta->SetXTitle(xtitle.c_str());
	Zprime_cosTheta->SetYTitle("#frac{1}{#sigma}#times#frac{d#sigma}{dcos#theta'}");

	dir_bjorken->cd();
	TH1 *Zprime_xBjorken_q = new TH1D("Zprime_xBjorken_q", "", xBnBins, xBmin, xBmax);
	Zprime_xBjorken_q->SetLineStyle(3);
	Zprime_xBjorken_q->SetXTitle("x_{q}");
	Zprime_xBjorken_q->SetYTitle("Events");
	
	TH1 *Zprime_xBjorken_qbar = new TH1D("Zprime_xBjorken_qbar", "", xBnBins, xBmin, xBmax);
	Zprime_xBjorken_qbar->SetLineStyle(3);
	Zprime_xBjorken_qbar->SetXTitle("x_{#bar q}");
	Zprime_xBjorken_qbar->SetYTitle("Events");




	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg_invMass = new TLegend(0.7,0.5,0.96,0.7);
	leg_invMass->AddEntry(SM_invMass,"SM","L");
	leg_invMass->AddEntry(KK_invMass,"KK","L");
	leg_invMass->AddEntry(Zprime_invMass,"Z'_{SM}","L");

	TLegend *leg_eta = new TLegend(0.7,0.7,0.96,0.9);
	leg_eta->AddEntry(SM_eta,"SM","lep");
	leg_eta->AddEntry(KK_eta,"KK","lep");
	leg_eta->AddEntry(Zprime_eta,"Z'_{SM}","lep");

	TLegend *leg_pT = new TLegend(0.7,0.7,0.96,0.9);
	leg_pT->AddEntry(SM_pT,"SM","L");
	leg_pT->AddEntry(KK_pT,"KK","L");
	leg_pT->AddEntry(Zprime_pT,"Z'_{SM}","L");

	TLegend *leg_cosTheta = NULL;
	leg_cosTheta = new TLegend(0.3,0.7,0.56,0.9);
	leg_cosTheta->AddEntry(SM_cosTheta,"SM","lep");
	leg_cosTheta->AddEntry(KK_cosTheta,"KK","lep");
	leg_cosTheta->AddEntry(Zprime_cosTheta,"Z'_{SM}","lep");
	
	TLegend *leg_bjorken_q = new TLegend(0.7,0.5,0.96,0.7);
	leg_bjorken_q->AddEntry(SM_xBjorken_q,"SM","L");
	leg_bjorken_q->AddEntry(KK_xBjorken_q,"KK","L");
	leg_bjorken_q->AddEntry(Zprime_xBjorken_q,"Z'_{SM}","L");

	TLegend *leg_bjorken_qbar = new TLegend(0.7,0.5,0.96,0.7);
	leg_bjorken_qbar->AddEntry(SM_xBjorken_qbar,"SM","L");
	leg_bjorken_qbar->AddEntry(KK_xBjorken_qbar,"KK","L");
	leg_bjorken_qbar->AddEntry(Zprime_xBjorken_qbar,"Z'_{SM}","L");

	TLegend *leg_QT = new TLegend(0.7,0.5,0.96,0.7);
	leg_QT->AddEntry(SM_QT,"SM","L");
	leg_QT->AddEntry(KK_QT,"KK","L");
	leg_QT->AddEntry(Zprime_QT,"Z'_{SM}","L");


	//----------------------------------------------------------------------------------------------------------
	
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double cosThetaBoost, cosThetaCollinsSoper;
	double invariantMass, pTransverse, Rapidity, cosTh=0, QTransverse;

	// to count the number of events in the KK peak area
	double KKpeak_highLimit = 5000.;
	double KKpeak_lowLimit = 2000.;
	double KKevents = 0;

	// to check the asymmetry in a narrow range arrownd the peak
	double mZ_known = physPrm.mStar;
	double mZ_lowLimit = physPrm.mStar-2000.;
	double mZ_highLimit = physPrm.mStar+1000.;
	cout << "\n" << "mZ_known=" << mZ_known << "  mZ_lowLimit=" << mZ_lowLimit << "  mZ_highLimit=" << mZ_highLimit << "\n" << endl;
	
	double Nbackward_SM = 0;
	double Nforward_SM = 0;
	double Nbackward_KK = 0;
	double Nforward_KK = 0;
	double Nbackward_ZE6 = 0;
	double Nforward_ZE6 = 0;
	double Afb, dAfb;




	// loop over 2 files (with / without Initial State Radiation)
	for (int f=1 ; f<=3 ; f++) {
		//-------------------------------------------------------------------------------------------------------------------
		// read the trees and put all good lepton pairs into 2 vectors
		//----- 1M
		if (sampleSizeSelector == 1) { //100k smaples
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_100000_EventsSample.root");
			}
			else if (f==2) { // KK
				if (nISRFSRswitch!=0) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_extractedParticles"+sISRFSRswitch+".root");
				}
				else if (nISRFSRswitch==0) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_100000_EventsSample.root");
				}
			}
			else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_100000_EventsSample.root");
			}
		}

		//----- 500
		else if (sampleSizeSelector == 2) { // 500[fb^-1]
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_extractedParticles_2trees_500inversefemtobarnSample.root");
			}
                        else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_extractedParticles_2trees_500inversefemtobarnSample.root");

                        }
                        else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_extractedParticles_2trees_500inversefemtobarnSample.root");
			}
		}

		//----- 100
		else if (sampleSizeSelector == 3) { // 100[fb^-1]
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_100inversefemtobarnSample.root");
				if(!isSmeared)
				{
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_true_extractedParticles_2trees_100inversefemtobarnSample.root");
				}
			}
			else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_100inversefemtobarnSample.root");
				if(!isSmeared)
				{
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_true_extractedParticles_2trees_100inversefemtobarnSample.root");
				}
			}
			else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_100inversefemtobarnSample.root");
				if(!isSmeared)
				{
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_true_extractedParticles_2trees_100inversefemtobarnSample.root");
				}
			}
		}

		Nevents = anlz.loopOverEvents(filePath, treeType.c_str());
		cout << "Nevents = " << Nevents << endl;
		// get all the good pairs (after cuts)
		if (!plotOnlyTrue){
			Nleptons = anlz.getAllLeptons(allgoodLeptons);
			Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
		}
		else if(plotOnlyTrue) {
			Nleptons  = anlz.getAllLeptonsTrue(allgoodLeptons);
	                Naleptons = anlz.getAllaLeptonsTrue(allgoodaLeptons);
		}
		cout << "Nleptons = " << Nleptons << endl;
		if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
		cout << "\n" << endl;
		//-------------------------------------------------------------------------------------------------------------------
	
		// run over the good pairs and fill the histograms
		for (int k=0 ; k<Nleptons ; k++) {
			invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
			QTransverse = anlz.pT(allgoodLeptons[k], allgoodaLeptons[k]);
			pTransverse = allgoodLeptons[k].pT;
			Rapidity = anlz.y(allgoodLeptons[k], allgoodaLeptons[k]);
			cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
			if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
			else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
			// fill the histos
			if (f==1) {
				SM_invMass->Fill( (Double_t)invariantMass );
				SM_pT->Fill( (Double_t)pTransverse );
				SM_QT->Fill( (Double_t)QTransverse );
				SM_eta->Fill( (Double_t)allgoodLeptons[k].eta );
				//SM_cosTheta->Fill( (Double_t)cosTh );
				if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
					SM_cosTheta->Fill( (Double_t)cosTh );
					if (cosTh>=-1. && cosTh<0.) {Nbackward_SM++;}
					else if (cosTh>=0. && cosTh<1.) {Nforward_SM++;}
					/*
					if (abs(cosTh) <= cosThMax) {
						costhetaCS = (Double_t)cosTh;
						costhetaCSmax = (Double_t)cuts.getMinCosTheta(invariantMass*invariantMass,Rapidity,runPrm); // !!!!!!!!!!!
						tree_SM->Fill();
					}
					*/
				}
				SM_xBjorken_q->Fill(allgoodLeptons[k].xq);
				SM_xBjorken_qbar->Fill(allgoodLeptons[k].xqbar);
			}
			else if (f==2) {
				KK2d_QT->Fill( (Double_t)invariantMass, (Double_t)QTransverse );
				KK_invMass->Fill( (Double_t)invariantMass );
				KK_pT->Fill( (Double_t)pTransverse );
				KK_QT->Fill( (Double_t)QTransverse );
				KK_eta->Fill( (Double_t)allgoodLeptons[k].eta );
				//KK_cosTheta->Fill( (Double_t)cosTh );
				if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
					KK_cosTheta->Fill( (Double_t)cosTh );
					if (cosTh>=-1. && cosTh<0.) {Nbackward_KK++;}
					else if (cosTh>=0. && cosTh<1.) {Nforward_KK++;}
					/*
					if (abs(cosTh) <= cosThMax) {
						costhetaCS = (Double_t)cosTh;
						costhetaCSmax = (Double_t)cuts.getMinCosTheta(invariantMass*invariantMass,Rapidity,runPrm); // !!!!!!!!!!
						tree_KK->Fill();
					}
					*/
				}
				if (invariantMass >= KKpeak_lowLimit && invariantMass <= KKpeak_highLimit) {KKevents++;}
				KK_xBjorken_q->Fill(allgoodLeptons[k].xq);
				KK_xBjorken_qbar->Fill(allgoodLeptons[k].xqbar);
			}
			else if (f==3) {
				Zprime_invMass->Fill( (Double_t)invariantMass );
				Zprime_pT->Fill( (Double_t)pTransverse );
				Zprime_QT->Fill( (Double_t)QTransverse );
				Zprime_eta->Fill( (Double_t)allgoodLeptons[k].eta );
				//Zprime_cosTheta->Fill( (Double_t)cosTh );
				if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
					Zprime_cosTheta->Fill( (Double_t)cosTh );
					if (cosTh>=-1. && cosTh<0.) {Nbackward_ZE6++;}
					else if (cosTh>=0. && cosTh<1.) {Nforward_ZE6++;}
					/*
					if (abs(cosTh) <= cosThMax) {
						costhetaCS = (Double_t)cosTh;
						costhetaCSmax = (Double_t)cuts.getMinCosTheta(invariantMass*invariantMass,Rapidity,runPrm); // !!!!!!!!!!
						tree_E6->Fill();
					}
					*/
				}
				Zprime_xBjorken_q->Fill(allgoodLeptons[k].xq);
				Zprime_xBjorken_qbar->Fill(allgoodLeptons[k].xqbar);
			}
		}
		// clear the pairs vectors
		anlz.clearAllLeptons();
		if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
		if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	}
	/*
	roo_treeFile->Write();
	*/

/*
	// normalize the cos(theta) distributions
	Double_t normalization = 1.;
	normalization = SM_cosTheta->Integral();
	if (normalization != 0) { SM_cosTheta->Scale(1./normalization); }
	else { cerr << "normalization for SM_cosTheta == 0" << endl; }
	normalization = SM_eta->Integral();
	if (normalization != 0) { SM_eta->Scale(1./normalization); }
	else { cerr << "normalization for SM_eta == 0" << endl; }
	
	normalization = Zprime_cosTheta->Integral();
	if (normalization != 0) { Zprime_cosTheta->Scale(1./normalization); }
	else { cerr << "normalization for Zprime_cosTheta == 0" << endl; }
	normalization = Zprime_eta->Integral();
	if (normalization != 0) { Zprime_eta->Scale(1./normalization); }
	else { cerr << "normalization for Zprime_eta == 0" << endl; }
	
	normalization = KK_cosTheta->Integral();
	if (normalization != 0) { KK_cosTheta->Scale(1./normalization); }
	else { cerr << "normalization for KK_cosTheta == 0" << endl; }
	normalization = KK_eta->Integral();
	if (normalization != 0) { KK_eta->Scale(1./normalization); }
	else { cerr << "normalization for KK_eta == 0" << endl; }
*/
	

	Double_t SM_norm = SM_cosTheta->GetEntries()*SM_cosTheta->GetBinWidth(2);

        commonOps.Norm((TH1D*)SM_cosTheta);
        commonOps.NormToBinWidth((TH1D*)SM_cosTheta);
        commonOps.Norm((TH1D*)SM_eta);
        commonOps.NormToBinWidth((TH1D*)SM_eta);

        commonOps.Norm((TH1D*)Zprime_cosTheta);
        commonOps.NormToBinWidth((TH1D*)Zprime_cosTheta);
        commonOps.Norm((TH1D*)Zprime_eta);
        commonOps.NormToBinWidth((TH1D*)Zprime_eta);

        commonOps.Norm((TH1D*)KK_cosTheta);
        commonOps.NormToBinWidth((TH1D*)KK_cosTheta);
        commonOps.Norm((TH1D*)KK_eta);
        commonOps.NormToBinWidth((TH1D*)KK_eta);


        KK_cosTheta     = commonOps.Shift((TH1D*)KK_cosTheta,0.1); // beware!! old hKK has leaked!!
        KK_eta          = commonOps.Shift((TH1D*)KK_eta,0.1); // beware!! old hKK has leaked!!
        Zprime_cosTheta = commonOps.Shift((TH1D*)Zprime_cosTheta,-0.1); // beware!!
        Zprime_eta      = commonOps.Shift((TH1D*)Zprime_eta,-0.1); // beware!!


/*
	vector<TArrow*> SM_zeros_cosTheta;
	TAxis *SM_xaxis_cosTheta = SM_cosTheta->GetXaxis();
	for (Int_t i=1 ; i>=SM_cosTheta->GetNbinsX() ; i++) {
		if (SM_cosTheta->GetBinContent(i)==0.) {
			Double_t binCenter = SM_xaxis_cosTheta->GetBinCenter(i);
			Double_t binHeight = 1.14/( SM_cosTheta->GetEntries()*SM_cosTheta->GetBinWidth(1) );
			TArrow* temp = new TArrow(binCenter, binHeight, binCenter, 0., 0.05, "|->");
			SM_zeros_cosTheta.push_back(temp);
		}
	}
*/
	cout << "\n" << endl;


	
	//--------------------------------------------------------------------------------------
	// calculate AFB SM:
	Afb = (Nforward_SM-Nbackward_SM)/(Nforward_SM+Nbackward_SM);
	dAfb = sqrt((1-Afb*Afb)/(Nforward_SM+Nbackward_SM));
	cout << "Afb(SM)[" << mZ_lowLimit << "<mHat<" << mZ_highLimit << "] = " << Afb << " +/-" << dAfb << "\t\tfor " << Nforward_SM+Nbackward_SM << " events in that range" << endl;
	//-----------------

	// calculate AFB Z':
	Afb = (Nforward_ZE6-Nbackward_ZE6)/(Nforward_ZE6+Nbackward_ZE6);
	dAfb = sqrt((1-Afb*Afb)/(Nforward_ZE6+Nbackward_ZE6));
	cout << "Afb(Z')[" << mZ_lowLimit << "<mHat<" << mZ_highLimit << "] = " << Afb << " +/-" << dAfb << "\t\tfor " << Nforward_ZE6+Nbackward_ZE6 << " events in that range" << endl;
	//-----------------

	// calculate AFB KK
	Afb = (Nforward_KK-Nbackward_KK)/(Nforward_KK+Nbackward_KK);
	dAfb = sqrt((1-Afb*Afb)/(Nforward_KK+Nbackward_KK));	
	cout << "Afb(KK)[" << mZ_lowLimit << "<mHat<" << mZ_highLimit << "] = " << Afb << " +/-" << dAfb << "\t\tfor " << Nforward_KK+Nbackward_KK << " events in that range" << endl;
	//-----------------
	cout << "\n" << endl;
	//--------------------------------------------------------------------------------------


/*
//	string infile_sigmaTheory = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"efficiency.root");
	string infile_sigmaTheory = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"efficiencies_mHat.root");
	TFile *treeFile = new TFile(infile_sigmaTheory.c_str());

	// get the data from the tree
	double mHat_SM;
	double mHat_KK;
	double mHat_E6;
	double sigma_SM;
	double sigma_KK;
	double sigma_E6;

	TTree *smTree = (TTree*)treeFile->Get("smTree");
	smTree->SetBranchAddress("mHat",&mHat_SM);
	smTree->SetBranchAddress("dsigmadmHat",&sigma_SM);
	Int_t treeNentries_SM = (Int_t)smTree->GetEntries();

	TTree *kkTree = (TTree*)treeFile->Get("kkTree");
	kkTree->SetBranchAddress("mHat",&mHat_KK);
	kkTree->SetBranchAddress("dsigmadmHat",&sigma_KK);
	Int_t treeNentries_KK = (Int_t)kkTree->GetEntries();

	TTree *e6Tree = (TTree*)treeFile->Get("e6Tree");
	e6Tree->SetBranchAddress("mHat",&mHat_E6);
	e6Tree->SetBranchAddress("dsigmadmHat",&sigma_E6);
	Int_t treeNentries_E6 = (Int_t)e6Tree->GetEntries();
	
	// calculate expected number of events in the peak area
	Double_t max_mZ = invMmax;
	Double_t min_mZ = invMmin;
	Double_t dmHat = 1.;

	Int_t nGraphEntries = (Int_t)( (max_mZ - min_mZ)/dmHat );
	Double_t *m = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	Double_t *sigma_SM_array = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	Double_t *sigma_KK_array = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	Double_t *sigma_E6_array = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));

	if (treeNentries_SM!=treeNentries_KK || treeNentries_SM!=treeNentries_E6 || treeNentries_KK!=treeNentries_E6) {
		cerr << "error: treeNentries_SM!=treeNentries_KK!=treeNentries_E6" << endl;
		exit(-1);
	}

	Int_t n=0;
	for (Int_t i=0; i<treeNentries_SM ; i++) {
		
		smTree->GetEntry(i);
		kkTree->GetEntry(i);
		e6Tree->GetEntry(i);

		if (mHat_SM!=mHat_KK || mHat_SM!=mHat_E6 || mHat_KK!=mHat_E6) {
			cerr << "error: mHat_SM!=mHat_KK!=mHat_E6" << endl;
			exit(-1);
		}

		if (mHat_SM>=min_mZ && mHat_SM<=max_mZ) {
			while (n<nGraphEntries && (mHat_SM>=min_mZ && mHat_SM<=max_mZ) && ((i+n)<treeNentries_SM)) {
				smTree->GetEntry(i+n);
				kkTree->GetEntry(i+n);
				e6Tree->GetEntry(i+n);

				m[n] = mHat_SM;
				sigma_SM_array[n] = sigma_SM;
				sigma_KK_array[n] = sigma_KK;
				sigma_E6_array[n] = sigma_E6;
				n++;
			}
		}
	}

	// calculate the integral of the theoretical line (simpson integration method)
	int nSegments = (int)nGraphEntries-1;
	Double_t h = dmHat/3;

	Double_t integrand_SM = 0;
	Double_t integrand_KK = 0;
	Double_t integrand_E6 = 0;

	Double_t sigmaTot_SM = 0;
	Double_t sigmaTot_KK = 0;
	Double_t sigmaTot_E6 = 0;

	Double_t sigmaTot_SM_peak = 0;
	Double_t sigmaTot_KK_peak = 0;
	Double_t sigmaTot_E6_peak = 0;

	int i=0;
	double mass = min_mZ;
	while (i<=nSegments) {
		// calculate the integrand:
		integrand_SM = sigma_SM_array[i];
		integrand_KK = sigma_KK_array[i];
		integrand_E6 = sigma_E6_array[i];
		// simpson:
		if (i==0 || i==nSegments) {
			sigmaTot_SM += integrand_SM;
			sigmaTot_KK += integrand_KK;
			sigmaTot_E6 += integrand_E6;
			if (mass>=KKpeak_lowLimit && mass<=KKpeak_highLimit) {
				sigmaTot_SM_peak += integrand_SM;
				sigmaTot_KK_peak += integrand_KK;
				sigmaTot_E6_peak += integrand_E6;
			}
		}
		else if (i%2==0 && i!=0 && i!=nSegments) {
			sigmaTot_SM += 2*integrand_SM;
			sigmaTot_KK += 2*integrand_KK;
			sigmaTot_E6 += 2*integrand_E6;
			if (mass>=KKpeak_lowLimit && mass<=KKpeak_highLimit) {
				sigmaTot_SM_peak += 2*integrand_SM;
				sigmaTot_KK_peak += 2*integrand_KK;
				sigmaTot_E6_peak += 2*integrand_E6;
			}
		}
		else {
			sigmaTot_SM += 4*integrand_SM;
			sigmaTot_KK += 4*integrand_KK;
			sigmaTot_E6 += 4*integrand_E6;
			if (mass>=KKpeak_lowLimit && mass<=KKpeak_highLimit) {
				sigmaTot_SM_peak += 4*integrand_SM;
				sigmaTot_KK_peak += 4*integrand_KK;
				sigmaTot_E6_peak += 4*integrand_E6;
			}
		}
		// propagate i:
		i++;
		mass += dmHat;
	}
	sigmaTot_SM *= h; // [nb]
	sigmaTot_KK *= h; // [nb]
	sigmaTot_E6 *= h; // [nb]

	sigmaTot_SM_peak *= h; // [nb]
	sigmaTot_KK_peak *= h; // [nb]
	sigmaTot_E6_peak *= h; // [nb]

	Double_t nb2mb = 1e-6;

	sigmaTot_SM *= nb2mb; // [mb]
	sigmaTot_KK *= nb2mb; // [mb]
	sigmaTot_E6 *= nb2mb; // [mb]

	sigmaTot_SM_peak *= nb2mb; // [mb]
	sigmaTot_KK_peak *= nb2mb; // [mb]
	sigmaTot_E6_peak *= nb2mb; // [mb]


	cout << "\n" << "sigma total = SM:" << sigmaTot_SM/nb2mb << "nb, Z':" << sigmaTot_E6/nb2mb << "nb, KK:" << sigmaTot_KK/nb2mb << "nb" << endl;
	cout << "\n" << "sigma peak = SM:" << sigmaTot_SM_peak/nb2mb << "nb, Z'" << sigmaTot_E6_peak/nb2mb << "nb, KK:" << sigmaTot_KK_peak/nb2mb << "nb" << endl;

	
	double luminosity = 100e+12;
	int luminositySelector = 1;
	cout << "choose luminosity: L=100[fb^-1](1) or  L=500[fb^-1](2)..." << endl;
	cin >> luminositySelector;
	if (luminositySelector!=1 && luminositySelector!=2) {
		cout << "invalid option, setting to default(1)" << endl;
	}
	else if (luminositySelector==2) {
		luminosity = 500e+12;
	}
	cout << "you choose L=" << luminosity/1e12 << "[1/fb]" << endl;

	cout << "\n" << "for: sigmaTot_SM=" << sigmaTot_SM << "[mb]" << "  and L=" << luminosity/1e12 << "[1/fb], we expect " << sigmaTot_SM*luminosity << " events between 1TeV and 6TeV" << endl;
	cout << "for: sigmaPeak_SM=" << sigmaTot_SM_peak << "[mb]" << "  and L=" << luminosity/1e12 << "[1/fb], we expect " << sigmaTot_SM_peak*luminosity << " events between 2TeV and 5TeV" << endl;

	cout << "\n" << "for: sigmaTot_KK=" << sigmaTot_KK << "[mb]" << "  and L=" << luminosity/1e12 << "[1/fb], we expect " << sigmaTot_KK*luminosity << " events between 1TeV and 6TeV" << endl;
	cout << "for: sigmaPeak_KK=" << sigmaTot_KK_peak << "[mb]" << "  and L=" << luminosity/1e12 << "[1/fb], we expect " << sigmaTot_KK_peak*luminosity << " events between 2TeV and 5TeV" << endl;

	cout << "\n" << "for: sigmaTot_E6=" << sigmaTot_E6 << "[mb]" << "  and L=" << luminosity/1e12 << "[1/fb], we expect " << sigmaTot_E6*luminosity << " events between 1TeV and 6TeV" << endl;
	cout << "for: sigmaPeak_E6=" << sigmaTot_E6_peak << "[mb]" << "  and L=" << luminosity/1e12 << "[1/fb], we expect " << sigmaTot_E6_peak*luminosity << " events between 2TeV and 5TeV" << endl;


	Int_t Ntot = (Int_t)KK_invMass->GetEntries(); // [events]

	
	double sigma_pythia = 1.800e-12; // [mb]
	cout << "\n" << "sigma_pythia=" << sigma_pythia << "[mb]" << " for " << Ntot << " events" << endl;
	double luminosity_pythia = Ntot/sigma_pythia; // [events/mb]
	cout << "luminosity_pythia=" << luminosity_pythia << "[mb^-1]" << "\n" << endl;
	double nKKpeakExpected = sigma_pythia * luminosity_pythia * (sigmaTot_KK_peak/sigmaTot_KK);
	cout << "from Pythia values: nKKpeakExpected = " << nKKpeakExpected << "  at the KK peak between 2TeV and 5TeV" << "\n" << endl;

	cout << "\n" << "number of events received in the peak area [" << KKpeak_lowLimit << "," << KKpeak_highLimit <<"] is: KKevents = " << KKevents << endl;
	//---------------------------------------------------------------------------------------------------------
*/

	// scale reference histos
	// for 1M KK events, if the SM sample size is also 1M then the histos should be scaled by 1.2
	// for 1M KK events, if the E6 sample size is also 1M then the histos should be scaled by 1.15
	if (sampleSizeSelector==1) {

		SM_invMass->Scale(1.2);
		SM_pT->Scale(1.2);
		SM_xBjorken_q->Scale(1.2);
		SM_xBjorken_qbar->Scale(1.2);

		Zprime_invMass->Scale(1.15);
		Zprime_pT->Scale(1.15);
		Zprime_xBjorken_q->Scale(1.15);
		Zprime_xBjorken_qbar->Scale(1.15);
	}



	dir_kinematics->cd();
        canv_kinematics->Draw();
    	canv_kinematics->cd();	
	canv_kinematics->cd(1);
	SM_invMass->Draw();
	//SM_invMass->DrawCopy("e1sames");
	Zprime_invMass->Draw("sames");
	KK_invMass->Draw("sames");
	//KK_invMass->DrawCopy("e1sames");
	leg_invMass->Draw("sames");
/*
	canv_kinematics->Update();
        TPaveStats *s1 = (TPaveStats*)Zprime_invMass->FindObject("stats");
        s1->SetY1NDC(0.818); //new y start position
        s1->SetY2NDC(0.9); //new y end position
	TPaveStats *s11 = (TPaveStats*)SM_invMass->FindObject("stats");
	s11->SetY1NDC(0.718); //new y start position
	s11->SetY2NDC(0.8); //new y end position
*/

	canv_kinematics->cd(2);
	gStyle->SetErrorX(0.);
	SM_eta->SetMarkerStyle(24);
	SM_eta->SetMarkerColor(kBlue);
	SM_eta->SetLineColor(kBlue);
        Zprime_eta->SetMarkerStyle(3);
        KK_eta->SetMarkerStyle(21);
	KK_eta->SetMarkerColor(kRed);
	KK_eta->SetLineColor(kRed);
	SM_eta->SetMaximum(1.4*SM_eta->GetMaximum());
	SM_eta->Draw("e1");
	//SM_eta->DrawCopy("e1sames");
	Zprime_eta->DrawCopy("e1sames");
	KK_eta->DrawCopy("e1sames");
	//KK_eta->DrawCopy("e1sames");
	leg_eta->Draw("sames");
/*
	canv_kinematics->Update();
        TPaveStats *s2 = (TPaveStats*)Zprime_eta->FindObject("stats");
        s2->SetY1NDC(0.818); //new y start position
        s2->SetY2NDC(0.9); //new y end position
	TPaveStats *s22 = (TPaveStats*)SM_eta->FindObject("stats");
	s22->SetY1NDC(0.718); //new y start position
	s22->SetY2NDC(0.8); //new y end position
*/

	canv_kinematics->cd(3);
	SM_pT->Draw();
	//SM_pT->DrawCopy("e1sames");
	Zprime_pT->Draw("sames");
	KK_pT->Draw("sames");
        //KK_pT->DrawCopy("e1sames");
	leg_pT->Draw("sames");
/*
	canv_kinematics->Update();
	TPaveStats *s3 = (TPaveStats*)Zprime_pT->FindObject("stats");
	s3->SetY1NDC(0.818); //new y start position
	s3->SetY2NDC(0.9); //new y end position
	TPaveStats *s33 = (TPaveStats*)SM_pT->FindObject("stats");
	s33->SetY1NDC(0.718); //new y start position
	s33->SetY2NDC(0.8); //new y end position
*/
	canv_kinematics->cd(4);
	gStyle->SetErrorX(0.);
	SM_cosTheta->SetMarkerStyle(24);
	SM_cosTheta->SetMarkerColor(kBlue);
	SM_cosTheta->SetLineColor(kBlue);
        Zprime_cosTheta->SetMarkerStyle(3);
        KK_cosTheta->SetMarkerStyle(21);
	KK_cosTheta->SetMarkerColor(kRed);
	KK_cosTheta->SetLineColor(kRed);
	if (sampleSizeSelector==1) {
		SM_cosTheta->SetMaximum(1.1*SM_cosTheta->GetMaximum());
	}
	else {
		SM_cosTheta->SetMaximum(1.6*SM_cosTheta->GetMaximum());
	}
	SM_cosTheta->Draw("e1x0");
	//SM_cosTheta->DrawCopy("e1sames");
	Zprime_cosTheta->DrawCopy("e1x0sames");
	KK_cosTheta->DrawCopy("e1x0sames");
	//KK_cosTheta->DrawCopy("e1sames");
	TAxis *SM_xaxis_cosTheta = SM_cosTheta->GetXaxis();
	TLine* SM_zeros_vert[(const int)(cosThetanBins+1)];
	TLine* SM_zeros_horiz[(const int)(cosThetanBins+1)];
	TMarker* SM_zeros_marker[(const int)(cosThetanBins+1)];
	for(int i=0 ; i<(int)(cosThetanBins+1) ; i++) {
		SM_zeros_vert[i] = NULL;
		SM_zeros_horiz[i] = NULL;
		SM_zeros_marker[i] = NULL;
	}
	
	if (sampleSizeSelector==3 /*&& !isSmeared*/) {
		//TLine* SM_zeros_vert[(const int)(SM_cosTheta->GetNbinsX()+1)];
		//TLine* SM_zeros_horiz[(const int)(SM_cosTheta->GetNbinsX()+1)];
		//TMarker* SM_zeros_marker[(const int)(SM_cosTheta->GetNbinsX()+1)];
		for(int i=1 ; i<=cosThetanBins ; i++)
		{
			if(SM_cosTheta->GetBinContent(i)==0)
			{
				Double_t binCenter = SM_xaxis_cosTheta->GetBinCenter(i);
				Double_t binHeight = 1.14/SM_norm;
				//TLine* SM_zeros_vert = new TArrow(binCenter, binHeight, binCenter, 0.);
				SM_zeros_vert[i] = new TArrow(binCenter, binHeight, binCenter, 0.); 
				SM_zeros_vert[i]->SetLineWidth(2);
				SM_zeros_vert[i]->SetLineColor(kBlue);
				//SM_zeros_vert[i]->Draw("sames");
				//TLine* SM_zeros_horiz = new TArrow(binCenter-0.025, binHeight, binCenter+0.025, binHeight);
				SM_zeros_horiz[i] = new TArrow(binCenter-0.025, binHeight, binCenter+0.025, binHeight);
	        		SM_zeros_horiz[i]->SetLineWidth(2);
				SM_zeros_horiz[i]->SetLineColor(kBlue);
				//SM_zeros_horiz[i]->Draw("sames");
				//TMarker* SM_zeros_marker = new TMarker(binCenter, 0.03, 23);
				SM_zeros_marker[i] = new TMarker(binCenter, 0.03, 23);
				SM_zeros_marker[i]->SetMarkerColor(kBlue);
				//SM_zeros_marker->SetMarkerSize(1.1);
				//SM_zeros_marker[i]->Draw("sames");
			}
		}
	}
	for(int i=0 ; i<(int)(cosThetanBins+1) ; i++) {
		if (SM_zeros_vert[i] != NULL) SM_zeros_vert[i]->Draw("sames");
		if (SM_zeros_horiz[i] != NULL) SM_zeros_horiz[i]->Draw("sames");
		if (SM_zeros_marker[i] != NULL) SM_zeros_marker[i]->Draw("sames");
	}
	leg_cosTheta->Draw("sames");
/*
	canv_kinematics->Update();
	TPaveStats *s4 = (TPaveStats*)Zprime_cosTheta->FindObject("stats");
	s4->SetY1NDC(0.818); //new y start position
	s4->SetY2NDC(0.9); //new y end position
	TPaveStats *s44 = (TPaveStats*)SM_cosTheta->FindObject("stats");
	s44->SetY1NDC(0.718); //new y start position
	s44->SetY2NDC(0.8); //new y end position
*/

	canv_kinematics->Update();
        canv_kinematics->Write();


	
	dir_mHat->cd();
	canv_mHat->Draw();
	canv_mHat->cd();
	SM_invMass->Draw();
        Zprime_invMass->Draw("sames");
        KK_invMass->Draw("sames");
        leg_invMass->Draw("sames");
	canv_mHat->Update();
        canv_mHat->Write();

	dir_pT->cd();
        canv_pT->Draw();
        canv_pT->cd();
        SM_pT->Draw();
        Zprime_pT->Draw("sames");
        KK_pT->Draw("sames");
        leg_pT->Draw("sames");
	canv_pT->Update();
        canv_pT->Write();
	
	dir_eta->cd();
	gStyle->SetErrorX(0.);
        canv_eta->Draw();
        canv_eta->cd();
	SM_eta->Draw("e1");
        Zprime_eta->DrawCopy("e1sames");
        KK_eta->DrawCopy("e1sames");
        leg_eta->Draw("sames");
	canv_eta->Update();
        canv_eta->Write();

	dir_cosTheta->cd();
	gStyle->SetErrorX(0.);
        canv_cosTheta->Draw();
        canv_cosTheta->cd();
        SM_cosTheta->Draw("e1");
        Zprime_cosTheta->DrawCopy("e1sames");
        KK_cosTheta->DrawCopy("e1sames");
	for(int i=0 ; i<(int)(cosThetanBins+1) ; i++) {
                if (SM_zeros_vert[i] != NULL) SM_zeros_vert[i]->Draw("sames");
                if (SM_zeros_horiz[i] != NULL) SM_zeros_horiz[i]->Draw("sames");
                if (SM_zeros_marker[i] != NULL) SM_zeros_marker[i]->Draw("sames");
        }
        leg_cosTheta->Draw("sames");
	canv_cosTheta->Update();
        canv_cosTheta->Write();
	

	dir_QT->cd();
        canv_QT->Draw();
        canv_QT->cd();
        SM_QT->Draw();
        Zprime_QT->Draw("sames");
        KK_QT->Draw("sames");
        leg_QT->Draw("sames");
        canv_QT->Update();
        canv_QT->Write();


	dir_bjorken->cd();
        canv_bjorken->Draw();
        canv_bjorken->cd();
	canv_bjorken->cd(1);
	SM_xBjorken_q->Draw();
	Zprime_xBjorken_q->Draw("sames");
	KK_xBjorken_q->Draw("sames");
	leg_bjorken_q->Draw("sames");
/*
	canv_bjorken->Update();
	TPaveStats *z1 = (TPaveStats*)Zprime_xBjorken_q->FindObject("stats");
	z1->SetY1NDC(0.818); //new y start position
	z1->SetY2NDC(0.9); //new y end position
	TPaveStats *z11 = (TPaveStats*)SM_xBjorken_q->FindObject("stats");
	z11->SetY1NDC(0.718); //new y start position
	z11->SetY2NDC(0.8); //new y end position
*/

	canv_bjorken->cd(2);
	SM_xBjorken_qbar->Draw();
	Zprime_xBjorken_qbar->Draw("sames");
	KK_xBjorken_qbar->Draw("sames");
	leg_bjorken_qbar->Draw("sames");
/*
	canv_bjorken->Update();
	TPaveStats *z2 = (TPaveStats*)Zprime_xBjorken_qbar->FindObject("stats");
	z2->SetY1NDC(0.818); //new y start position
	z2->SetY2NDC(0.9); //new y end position
	TPaveStats *z22 = (TPaveStats*)SM_xBjorken_qbar->FindObject("stats");
	z22->SetY1NDC(0.718); //new y start position
	z22->SetY2NDC(0.8); //new y end position
*/
	canv_bjorken->Update();
	canv_bjorken->Write();





/*
	canv_QTvsMhat->cd();
	canv_QTvsMhat->cd(1);
	SM_QT->Draw();
	Zprime_QT->Draw("sames");
	KK_QT->Draw("sames");
	//KK_pT->DrawCopy("e1sames");
	leg_QT->Draw("sames");
	canv_QTvsMhat->Update();
	TPaveStats *w1 = (TPaveStats*)Zprime_QT->FindObject("stats");
	w1->SetY1NDC(0.818); //new y start position
	w1->SetY2NDC(0.9); //new y end position
	TPaveStats *w2 = (TPaveStats*)SM_QT->FindObject("stats");
	w2->SetY1NDC(0.718); //new y start position
	w2->SetY2NDC(0.8); //new y end position

	canv_QTvsMhat->cd(2);
*/

	
	dir_QTvsMhat->cd();
        canv_QTvsMhat->Draw();
        canv_QTvsMhat->cd();
	canv_QTvsMhat->cd();
	if (n2DPlot!=1) {
		gStyle->SetPalette(1);
	}
	if(n2DPlot==0) { KK2d_QT->Draw(); }
	else { KK2d_QT->Draw(s2DPlot.c_str()); }
	canv_QTvsMhat->Update();
	canv_QTvsMhat->Write();





	cout << "\n---finalizing---\n" << endl;

	kinematicsHistFile->Write();

	//canv_kinematics->Print(kinematicsRootFileName.c_str(),"root");
	canv_kinematics->Print(kinematicsEpsFileName.c_str());

	//canv_bjorken->Print(bjorkenRootFileName.c_str(),"root");
	canv_bjorken->Print(bjorkenEpsFileName.c_str());
	//bjorkenHistFile->Write();
	
	//canv_QTvsMhat->Print(QTrootFileName.c_str(),"root");
        canv_QTvsMhat->Print(QTepsFileName.c_str());
        //QThistFile->Write();

	// wait so the plots can hold on
	canv_kinematics->WaitPrimitive();
	canv_bjorken->WaitPrimitive();
	canv_QTvsMhat->WaitPrimitive();


	// Done.
	return 0;
}

