/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Moses1;



Double_t fitFunction(Double_t *x, Double_t *par) {
	Double_t fitfunc = 1. - par[0] * pow(x[0], par[1]);
	return fitfunc;
}



int main() {
	cout << "Holy Moses..." << endl;
	
	TStyle* plotstyle = (TStyle*)gROOT->GetStyle("Plain");
	plotstyle->cd();

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();


	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)


	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS}; // glu, gamma, Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here


	// create relevant classes instances
	couplings cplng; // set the class for couplings call
	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	lhapdf lha; // set the class for lhapdf call
	newZresonance zPrime("chi");
	bool doSMcouplings = true;
	//--------------------------------------------------------------------//


	// for the runs
	dcomplex fullAmp_SM (0,0);
	dcomplex fullAmp_KK (0,0);
	dcomplex fullAmp_Zprime (0,0);

	int nMax_ysegments = 100; // for the integration on the rapidity
	int nKKmodes = 100;

	double differentialSquareAmp_SM = 0;
	double differentialSquareAmp_KK = 0;
	double differentialSquareAmp_Zprime = 0;

	double differentialSquareAmp_SM_noCuts = 0;
	double differentialSquareAmp_KK_noCuts = 0;
	double differentialSquareAmp_Zprime_noCuts = 0;
	
	double squareAmp_SM = 0;
	double squareAmp_KK = 0;
	double squareAmp_Zprime = 0;
	
	double squareAmp_SM_noCuts = 0;
	double squareAmp_KK_noCuts = 0;
	double squareAmp_Zprime_noCuts = 0;	

	double totalwidth_ZKK = 0;
	double totalwidth_Z0 = 0;
	double totalwidth_Zprime = 0;
	double totalwidth_gammaKK = 0;

	double sigmaRatio_SM = 0;
	double sigmaRatio_KK = 0;
	double sigmaRatio_Zprime = 0;

	double qqbarRapidity_shat = 0; // this is the rapidity integral factor (single integral)
	double qqbarRapidity_shat_noCuts = 0;
	customParticle outFermion = CUSTOMfermions[7]; // propagator will decay into a muon muon_bar pair
	// the integral over all decay angles
	// [1] theta: distributing like ~ (1+/-cos(theta))^2
	// [2] phi: distributing uniformly
	double angularIntegral = 2*physPrm.pi;
	

	double mHat_min = 2000./*2000.*/;
	double mHat_max = 5000./*5000.*/;
	double dmHat = 100.;
	double mHat = mHat_min;
	double s_hat = mHat_min*mHat_min;
	double h=dmHat/3.;
	int nActualSegments = (int)( (mHat_max - mHat_min)/dmHat );
	double integrand_SM = 0.;
	double integrand_E6 = 0.;
	double integrand_KK = 0.;
	double I_SM = 0;
	double I_E6 = 0;
	double I_KK = 0;

	double integrand_SM_noCuts = 0.;
	double integrand_E6_noCuts = 0.;
	double integrand_KK_noCuts = 0.;
	double I_SM_noCuts = 0;
	double I_E6_noCuts = 0;
	double I_KK_noCuts = 0;

	int i=0;
	

	double cosTheta_min = -1.;
	double cosTheta_max = +1.; //0.986614298;
	double dcosTheta = 0.01;
	double cosTheta = cosTheta_min;
	Int_t nentries = (Int_t)((cosTheta_max-cosTheta_min)/dcosTheta);
	cout << "nentries=" << nentries << endl;

	// initialize lha (lhapdf instance)
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name
	
	// alocate arrays for the graph:
	Double_t *CosTheta = (Double_t *)malloc(nentries*sizeof(Double_t));
	
	Double_t *Sqare_amp_SM_noCuts     = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_KK_noCuts     = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_Zprime_noCuts = (Double_t *)malloc(nentries*sizeof(Double_t));

	Double_t *Sqare_amp_SM     = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_KK     = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_Zprime = (Double_t *)malloc(nentries*sizeof(Double_t));

	Double_t *ratio_SM     = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *ratio_KK     = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *ratio_Zprime = (Double_t *)malloc(nentries*sizeof(Double_t));

	
	// write to dat file
//	ofstream ouputFile;
//	string KKresultsFile = commonOps.checkANDsetFilepath("MOSESDATA","/sigma_cuts.KK");
//	ouputFile.open(KKresultsFile.c_str());


	string dirName = "/KKvsZE6vsSMstudy/";

	string treeFilePath = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"efficiencies_cosTheta.root");
	TFile *treeFile = new TFile(treeFilePath.c_str(),"RECREATE");

	string treeName = "smTree";
	TTree *smTree = new TTree(treeName.c_str(),treeName.c_str());
	treeName = "kkTree";
        TTree *kkTree = new TTree(treeName.c_str(),treeName.c_str());
	treeName = "e6Tree";
        TTree *e6Tree = new TTree(treeName.c_str(),treeName.c_str());

	smTree->Branch("cosTheta",&cosTheta,"cosTheta/D");
	smTree->Branch("dsigmadmHat_noCuts",&squareAmp_SM_noCuts,"squareAmp_SM_noCuts/D");
	smTree->Branch("dsigmadmHat",&squareAmp_SM,"squareAmp_SM/D");
	smTree->Branch("ratio",&sigmaRatio_SM,"sigmaRatio_SM/D");

	kkTree->Branch("cosTheta",&cosTheta,"cosTheta/D");
	kkTree->Branch("dsigmadmHat_noCuts",&squareAmp_KK_noCuts,"squareAmp_KK_noCuts/D");
	kkTree->Branch("dsigmadmHat",&squareAmp_KK,"squareAmp_KK/D");
	kkTree->Branch("ratio",&sigmaRatio_KK,"sigmaRatio_KK/D");

	e6Tree->Branch("cosTheta",&cosTheta,"cosTheta/D");
	e6Tree->Branch("dsigmadmHat_noCuts",&squareAmp_Zprime_noCuts,"squareAmp_Zprime_noCuts/D");
	e6Tree->Branch("dsigmadmHat",&squareAmp_Zprime,"squareAmp_Zprime/D");
	e6Tree->Branch("ratio",&sigmaRatio_Zprime,"sigmaRatio_Zprime/D");


	cout << "\n\nRun over cosTheta ([" << cosTheta_min << "->" << cosTheta_max << "]:" << endl;
	for (Int_t n=0 ; n<nentries ; n++) { // loop over cosTheta
		i=0;
		I_SM=0;
		I_E6=0;
		I_KK=0;
		I_SM_noCuts=0;
                I_E6_noCuts=0;
                I_KK_noCuts=0;
		mHat = mHat_min; // lower mHat
		s_hat = mHat*mHat;
		while (mHat<=mHat_max && i<=nActualSegments) {
			integrand_SM = 0.;
			integrand_E6 = 0.;
			integrand_KK = 0.;
			integrand_SM_noCuts = 0.;
                        integrand_E6_noCuts = 0.;
                        integrand_KK_noCuts = 0.;
	
			// before each new s_hat it needs to be initialized to '0'
			squareAmp_SM        = 0;
			squareAmp_SM_noCuts = 0;
			squareAmp_KK        = 0;
	                squareAmp_KK_noCuts = 0;
			squareAmp_Zprime        = 0;
	                squareAmp_Zprime_noCuts = 0;

			// before each new s_hat it needs to be initialized to '0'
			totalwidth_Z0 = 0;
			totalwidth_ZKK = 0;
			totalwidth_gammaKK = 0;
			totalwidth_Zprime = 0;

			// calculate the total width of Z/gamma* for each s_hat:
			// calculate the partial width to this pair of ffbar and summ it to get the total width
			for (int j=0 ; j<(int)CUSTOMfermions.size() ; j++) {
				totalwidth_Z0 += wdt.getpartialWidth(0, s_hat, PDTbosons[2], CUSTOMfermions[j], physPrm); // Z0 total width
				totalwidth_Zprime = zPrime.getZprimeWidth(doSMcouplings, CUSTOMfermions[j]); // Z'
			}
	
			for (int q=0 ; q<5 ; q++) { // run over all incoming quarks pairs (q+qbar) except for ttbar
				
				differentialSquareAmp_SM = 0; // before each new q it needs to be initialized to '0'
				differentialSquareAmp_KK = 0;
				differentialSquareAmp_Zprime = 0;

				for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
					for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states (muon only)
	
						fullAmp_SM     = dcomplex(0,0); // before each helicity state this has to be initialized to '(0,0)'
						fullAmp_KK     = dcomplex(0,0);
						fullAmp_Zprime = dcomplex(0,0);

						// ---------- SM terms ----------------------------------------------------------------------
						// photon propagator
						fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, 0, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[1], physPrm) * (1. + 4.*hIn*hOut*cosTheta);
						// Z0 propagator
						fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, totalwidth_Z0, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[2], physPrm) * (1. + 4.*hIn*hOut*cosTheta);
						// ------------------------------------------------------------------------------------------
	
						// ---------- KK terms ----------------------------------------------------------------------
						fullAmp_KK = fullAmp_SM;
						for (int KKmode=1 ; KKmode<=nKKmodes ; KKmode++) { // loop over all KK modes
							totalwidth_ZKK = 0.;
							totalwidth_gammaKK = 0.;
							for (int j=0 ; j<(int)CUSTOMfermions.size() ; j++) {
								totalwidth_ZKK+=wdt.getpartialWidth(KKmode,s_hat,PDTbosons[2],CUSTOMfermions[j],physPrm);//Z*
								switch (CUSTOMfermions[j].pdt_prt.id) {
									case PDTNUE: break;
									case PDTNUMU: break;
									case PDTNUTAU: break;
									default: // gamma can not decay into neutrino pair (while Z can)
										totalwidth_gammaKK += wdt.getpartialWidth(KKmode, s_hat,
												PDTbosons[1], CUSTOMfermions[j], physPrm); // gamma*
										break;
								}
							}
						
							// photon* propagator
							fullAmp_KK += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_gammaKK, hIn, hOut,
												CUSTOMfermions[q], outFermion,
												PDTbosons[1], physPrm);

							// Z* propagator
							fullAmp_KK += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_ZKK, hIn, hOut,
												CUSTOMfermions[q], outFermion,
												PDTbosons[2], physPrm);
						}
						fullAmp_KK *= (1. + 4.*hIn*hOut*cosTheta);
						// ------------------------------------------------------------------------------------------
									
						// ---------- Z' terms ----------------------------------------------------------------------
						fullAmp_Zprime = zPrime.getSquaredAmplitude(hIn,hOut,s_hat,CUSTOMfermions[q],outFermion,
										totalwidth_Z0,totalwidth_Zprime,doSMcouplings) * (1. + 4.*hIn*hOut*cosTheta);
						// ------------------------------------------------------------------------------------------
	
	
						// get the differential square amplitude: |fullAmp|^2
						differentialSquareAmp_SM     += (s_hat*s_hat) * (1./4.) * real( fullAmp_SM*conj(fullAmp_SM) );
						differentialSquareAmp_KK     += (s_hat*s_hat) * (1./4.) * real( fullAmp_KK*conj(fullAmp_KK) );
						differentialSquareAmp_Zprime += (s_hat*s_hat) * (1./4.) * real( fullAmp_Zprime*conj(fullAmp_Zprime) );

						differentialSquareAmp_SM_noCuts     += (s_hat*s_hat) * (1./4.) * real( fullAmp_SM*conj(fullAmp_SM) );
						differentialSquareAmp_KK_noCuts     += (s_hat*s_hat) * (1./4.) * real( fullAmp_KK*conj(fullAmp_KK) );
						differentialSquareAmp_Zprime_noCuts += (s_hat*s_hat) * (1./4.) * real( fullAmp_Zprime*conj(fullAmp_Zprime) );

					} // end hOut
				} // end hIn
			
				qqbarRapidity_shat        = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha,abs(cosTheta));//cuts
				qqbarRapidity_shat_noCuts = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha);//no cuts

				// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
				// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
				//     and here only 3 are valid sice the propogator is colorless
				// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams
				differentialSquareAmp_SM     *= 2*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
				differentialSquareAmp_KK     *= 2*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
				differentialSquareAmp_Zprime *= 2*qqbarRapidity_shat/CUSTOMfermions[q].Nc;

				differentialSquareAmp_SM_noCuts     *= 2*qqbarRapidity_shat_noCuts/CUSTOMfermions[q].Nc;
				differentialSquareAmp_KK_noCuts     *= 2*qqbarRapidity_shat_noCuts/CUSTOMfermions[q].Nc;
				differentialSquareAmp_Zprime_noCuts *= 2*qqbarRapidity_shat_noCuts/CUSTOMfermions[q].Nc;

				// only then add the differential square amplitude to the full one
				squareAmp_SM     += differentialSquareAmp_SM;
				squareAmp_KK     += differentialSquareAmp_KK;
				squareAmp_Zprime += differentialSquareAmp_Zprime;

				squareAmp_SM_noCuts     += differentialSquareAmp_SM_noCuts;
				squareAmp_KK_noCuts     += differentialSquareAmp_KK_noCuts;
				squareAmp_Zprime_noCuts += differentialSquareAmp_Zprime_noCuts;
			} // end q
	
        	        // multiply by the coefficients appearing before the squared helicity amplitude
                	squareAmp_SM     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
                	squareAmp_KK     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
                	squareAmp_Zprime *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
		
			squareAmp_SM_noCuts     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
			squareAmp_KK_noCuts     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
			squareAmp_Zprime_noCuts *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;

                	        // [squareAmp] = [1/GeV^4]
                	squareAmp_SM     *= 2*sqrt(s_hat); // dSigma(s_hat) -> dSigma(sqrt(s_hat)) Jacobian of the transformation
                	squareAmp_KK     *= 2*sqrt(s_hat);
                	squareAmp_Zprime *= 2*sqrt(s_hat);

			squareAmp_SM_noCuts     *= 2*sqrt(s_hat);
			squareAmp_KK_noCuts     *= 2*sqrt(s_hat);
			squareAmp_Zprime_noCuts *= 2*sqrt(s_hat);
                	        // [squareAmp] = [1/GeV^3]
                	squareAmp_SM     *= physPrm.GeV2nbarn; // to convert the units to nb/GeV (originally was nb/GeV^2 but here it is nb/GeV)
                	squareAmp_KK     *= physPrm.GeV2nbarn;
                	squareAmp_Zprime *= physPrm.GeV2nbarn;
		
			squareAmp_SM_noCuts     *= physPrm.GeV2nbarn;
			squareAmp_KK_noCuts     *= physPrm.GeV2nbarn;
			squareAmp_Zprime_noCuts *= physPrm.GeV2nbarn;
                	        // [squareAmp] = [nb/GeV]
                	// see more details about the conversion using the method "prm.printUnits()" in the end of the loop.

			integrand_SM = squareAmp_SM;
			integrand_E6 = squareAmp_Zprime;
			integrand_KK = squareAmp_KK;

			integrand_SM_noCuts = squareAmp_SM_noCuts;
			integrand_E6_noCuts = squareAmp_Zprime_noCuts;
			integrand_KK_noCuts = squareAmp_KK_noCuts;
				
				// simpson
				if (i==0 || i==nActualSegments) {
					if (s_hat >= 20*20/(1.-cosTheta*cosTheta)) {
						I_SM += integrand_SM;
						I_E6 += integrand_E6;
						I_KK += integrand_KK;
					}
					I_SM_noCuts += integrand_SM_noCuts;
					I_E6_noCuts += integrand_E6_noCuts;
					I_KK_noCuts += integrand_KK_noCuts;
				}
				else if (i%2==0 && i!=0 && i!=nActualSegments) {
					if (s_hat >= 20*20/(1.-cosTheta*cosTheta)) {
						I_SM += 2*integrand_SM;
						I_E6 += 2*integrand_E6;
						I_KK += 2*integrand_KK;
					}
					I_SM_noCuts += 2*integrand_SM_noCuts;
					I_E6_noCuts += 2*integrand_E6_noCuts;
					I_KK_noCuts += 2*integrand_KK_noCuts;
				}
				else {
					if (s_hat >= 20*20/(1.-cosTheta*cosTheta)) {
						I_SM += 4*integrand_SM;
						I_E6 += 4*integrand_E6;
						I_KK += 4*integrand_KK;
					}
					I_SM_noCuts += 4*integrand_SM_noCuts;
					I_E6_noCuts += 4*integrand_E6_noCuts;
					I_KK_noCuts += 4*integrand_KK_noCuts;
				}
			i++;
			cout << "mHat=" << mHat << "\r" << flush;
			mHat += dmHat;
			s_hat = mHat*mHat;
		}
		cout << endl;

                // fill arrays of graphs:
                Sqare_amp_SM[n]     = I_SM * h;
                Sqare_amp_KK[n]     = I_KK * h;
                Sqare_amp_Zprime[n] = I_E6 * h;

		Sqare_amp_SM_noCuts[n]     = I_SM_noCuts * h;
		Sqare_amp_KK_noCuts[n]     = I_KK_noCuts * h;
		Sqare_amp_Zprime_noCuts[n] = I_E6_noCuts * h;	
	
		sigmaRatio_SM     = I_SM/I_SM_noCuts;
		sigmaRatio_KK     = I_KK/I_KK_noCuts;
		sigmaRatio_Zprime = I_E6/I_E6_noCuts;

		ratio_SM[n]     = sigmaRatio_SM;
		ratio_KK[n]     = sigmaRatio_KK;
		ratio_Zprime[n] = sigmaRatio_Zprime;

		CosTheta[n] = cosTheta;

		smTree->Fill();
		kkTree->Fill();
		e6Tree->Fill();

		cout << "cosTheta=" << cosTheta << "   SM=" << sigmaRatio_SM << "   KK=" << sigmaRatio_KK << "   Z'=" << sigmaRatio_Zprime << "\r" << flush;
		cosTheta += dcosTheta;
	} // end cosTheta

	smTree->Print();
	smTree->Write();
	//smTree->Write(treeFilePath.c_str());
	kkTree->Print();
	kkTree->Write();
	//kkTree->Write(treeFilePath.c_str());
	e6Tree->Print();
	e6Tree->Write();
	//e6Tree->Write(treeFilePath.c_str());

	treeFile->Write();

	cout << "\n\n" << "--------------creating graph--------------" << endl;

	string graph = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"efficiencies_graphs_cosTheta.root");
	TFile *graphFile = new TFile(graph.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph

	TCanvas *c1 = new TCanvas("c1","pp->mu-mu+  dsigma/dsHat cuts influences",0,0,1400,800);
/*
	c1->Divide(1,2);
	
	TVirtualPad *pad1=c1->cd(1);
	pad1->SetLogy();
	pad1->SetFillColor(0);

	TVirtualPad *pad2=c1->cd(2);
	pad2->SetGrid();
	pad2->SetFillColor(0);

	c1->cd(1);
	TGraph *crossSection_SM_noCuts = new TGraph (nentries, CosTheta, Sqare_amp_SM_noCuts);
	TGraph *crossSection_SM = new TGraph (nentries, CosTheta, Sqare_amp_SM);
	TGraph *crossSection_KK_noCuts = new TGraph (nentries, CosTheta, Sqare_amp_KK_noCuts);
        TGraph *crossSection_KK = new TGraph (nentries, CosTheta, Sqare_amp_KK);
	TGraph *crossSection_Zprime_noCuts = new TGraph (nentries, CosTheta, Sqare_amp_Zprime_noCuts);
        TGraph *crossSection_Zprime = new TGraph (nentries, CosTheta, Sqare_amp_Zprime);

	crossSection_SM->SetTitle("pp->mu-mu+: cuts influence on digma/dsHat");
	crossSection_SM->SetMaximum(1e-5);
	crossSection_SM->SetLineStyle(1);
	crossSection_SM->SetLineColor(kRed);
	crossSection_SM->GetXaxis()->SetTitle("cos(theta*)");
	crossSection_SM->GetYaxis()->SetTitle("differential cross section [nb/GeV]");
	crossSection_SM->Draw("AC");
	crossSection_SM_noCuts->SetLineStyle(2);
	crossSection_SM_noCuts->SetLineColor(kRed);
	crossSection_SM_noCuts->Draw("C");

	crossSection_KK->SetLineStyle(1);
	crossSection_KK->SetLineColor(kBlack);
	crossSection_KK->Draw("C");
	crossSection_KK_noCuts->SetLineStyle(2);
	crossSection_KK_noCuts->SetLineColor(kBlack);
        crossSection_KK_noCuts->Draw("C");

	crossSection_Zprime->SetLineStyle(1);
	crossSection_Zprime->SetLineColor(kGreen);
	crossSection_Zprime->Draw("C");
	crossSection_Zprime_noCuts->SetLineStyle(2);
	crossSection_Zprime_noCuts->SetLineColor(kGreen);
	crossSection_Zprime_noCuts->Draw("C");

	TLegend *leg = new TLegend(0.75, 0.5, 0.90, 0.90);
	leg->SetFillColor(5);
	leg->AddEntry(crossSection_SM_noCuts,"SM without cuts","L");
	leg->AddEntry(crossSection_SM,"SM with cuts","L");
	leg->AddEntry(crossSection_KK_noCuts,"KK without cuts","L");
        leg->AddEntry(crossSection_KK,"KK with cuts","L");
	leg->AddEntry(crossSection_Zprime_noCuts,"Z' without cuts","L");
        leg->AddEntry(crossSection_Zprime,"Z' with cuts","L");
	
	leg->Draw();
	//----------------------


	c1->cd(2);	
*/
	TGraph *ratio_graph_SM = new TGraph (nentries, CosTheta, ratio_SM);
	TGraph *ratio_graph_KK = new TGraph (nentries, CosTheta, ratio_KK);
	TGraph *ratio_graph_Zprime = new TGraph (nentries, CosTheta, ratio_Zprime);

//	ratio_graph_SM->SetTitle("qqbar->mu-mu+: efficiency(cosTheta*)");
	ratio_graph_SM->SetTitle("");
	ratio_graph_SM->SetMaximum(1.25);
	ratio_graph_SM->SetLineStyle(5);
//	ratio_graph_SM->SetLineColor(kRed);
	ratio_graph_SM->GetXaxis()->SetTitle("cos#theta*_{#beta}");
	ratio_graph_SM->GetYaxis()->SetTitle("#left(#frac{d#sigma}{dcos#theta*_{#beta}}#right)_{cuts} #divide #left(#frac{d#sigma}{dcos#theta*_{#beta}}#right)_{full}");
	ratio_graph_SM->Draw("AC");

//	ratio_graph_KK->SetLineColor(kBlack);
	ratio_graph_KK->SetLineStyle(1);
	ratio_graph_KK->Draw("C");

//	ratio_graph_Zprime->SetLineColor(kGreen);
	ratio_graph_Zprime->SetLineStyle(3);
	ratio_graph_Zprime->Draw("C");


	stringstream stmTmp;
	string par1Tmp, par2Tmp;
	string fitFormula;
	Int_t power = 0;
	Int_t dpower = 0;
	Double_t norm = 1.;
	Double_t dnorm = 0.;
	Double_t chi2dof = 0.;
	
	//create a function in the range [-1,1] with 2 parameters
	TF1 *SM_func = new TF1("SM_efficiencyFit",fitFunction,-1.,1.,2);
	// initialize parameters
	SM_func->SetParameter(0,10.);
	SM_func->SetParameter(1,1.);
	SM_func->SetParNames("norm","power");
	// make the fit
	ratio_graph_SM->Fit("SM_efficiencyFit");
	// get the results
	TF1 *SM_fitResult = ratio_graph_SM->GetFunction("SM_efficiencyFit");
	norm = SM_fitResult->GetParameter(0);
	dnorm = SM_fitResult->GetParError(0);
	power = (Int_t)SM_fitResult->GetParameter(1);
        if (power%2 != 0) {
                cout << "cast to Int_t resulted odd power, fixing it to n --> n+1" << endl;
                power++;
        }
	dpower = (Int_t)SM_fitResult->GetParError(1);
	chi2dof = SM_fitResult->GetChisquare()/SM_fitResult->GetNDF();
	cout << "SM: norm=" << norm << "+/-" << dnorm << "\t" << "power=" << power << "+/-" << dpower << "\t" << "chi2dof=" << chi2dof << endl;
	stmTmp.clear();
	par1Tmp.clear();
	par2Tmp.clear();
	//----------
	stmTmp << norm;  stmTmp >> par1Tmp;
	stmTmp.clear();
	stmTmp << power; stmTmp >> par2Tmp;
	//----------
	fitFormula = "1 - " + par1Tmp + " * x^" + par2Tmp;
	cout << "fitFormula = " << fitFormula << endl;
	TF1 *fSM = new TF1("fSM", fitFormula.c_str(), -1.,1.);
	fSM->SetLineColor(kRed);
	fSM->SetLineWidth(1);
//	fSM->Draw("SAME");
//	SM_fitResult->Draw("SAME");

	

        //create a function in the range [-1,1] with 2 parameters
        TF1 *E6_func = new TF1("E6_efficiencyFit",fitFunction,-1.,1.,2);
        // initialize parameters
        E6_func->SetParameter(0,10.);
        E6_func->SetParameter(1,1.);
        E6_func->SetParNames("norm","power");
        // make the fit
        ratio_graph_Zprime->Fit("E6_efficiencyFit");
        // get the results
        TF1 *E6_fitResult = ratio_graph_Zprime->GetFunction("E6_efficiencyFit");
        norm = E6_fitResult->GetParameter(0);
        dnorm = E6_fitResult->GetParError(0);
        power = (Int_t)E6_fitResult->GetParameter(1);
	if (power%2 != 0) {
                cout << "cast to Int_t resulted odd power, fixing it to n --> n+1" << endl;
                power++;
        }
        dpower = (Int_t)E6_fitResult->GetParError(1);
        chi2dof = E6_fitResult->GetChisquare()/E6_fitResult->GetNDF();
        cout << "E6: norm=" << norm << "+/-" << dnorm << "\t" << "power=" << power << "+/-" << dpower << "\t" << "chi2dof=" << chi2dof << endl;
        stmTmp.clear();
        par1Tmp.clear();
	par2Tmp.clear();
	//----------
        stmTmp << norm;  stmTmp >> par1Tmp;
        stmTmp.clear();
        stmTmp << power; stmTmp >> par2Tmp;
        //----------
        fitFormula = "1 - " + par1Tmp + " * x^" + par2Tmp;
        cout << "fitFormula = " << fitFormula << endl;
        TF1 *fE6 = new TF1("fE6", fitFormula.c_str(), -1.,1.);
	fE6->SetLineStyle(2);
	fE6->SetLineWidth(1);
        fE6->SetLineColor(kGreen);
//        fE6->Draw("SAME");


        //create a function in the range [-1,1] with 2 parameters
        TF1 *KK_func = new TF1("KK_efficiencyFit",fitFunction,-1.,1.,2);
        // initialize parameters
        KK_func->SetParameter(0,10.);
        KK_func->SetParameter(1,1.);
        KK_func->SetParNames("norm","power");
        // make the fit
        ratio_graph_KK->Fit("KK_efficiencyFit");
        // get the results
        TF1 *KK_fitResult = ratio_graph_KK->GetFunction("KK_efficiencyFit");
        norm = KK_fitResult->GetParameter(0);
        dnorm = KK_fitResult->GetParError(0);
        power = (Int_t)KK_fitResult->GetParameter(1);
	if (power%2 != 0) {
		cout << "cast to Int_t resulted odd power, fixing it to n --> n+1" << endl;
		power++;
	}
        dpower = (Int_t)KK_fitResult->GetParError(1);
        chi2dof = KK_fitResult->GetChisquare()/KK_fitResult->GetNDF();
        cout << "KK: norm=" << norm << "+/-" << dnorm << "\t" << "power=" << power << "+/-" << dpower << "\t" << "chi2dof=" << chi2dof << endl;
        stmTmp.clear();
        par1Tmp.clear();
        par2Tmp.clear();
	//----------
        stmTmp << norm;  stmTmp >> par1Tmp;
        stmTmp.clear();
        stmTmp << power; stmTmp >> par2Tmp;
        //----------
        fitFormula = "1 - " + par1Tmp + " * x^" + par2Tmp;
        cout << "fitFormula = " << fitFormula << endl;
        TF1 *fKK = new TF1("fKK", fitFormula.c_str(), -1.,1.);
        fKK->SetLineColor(kBlack);
	fKK->SetLineWidth(1);
//      fKK->Draw("SAME");



	// simple fit
//	TF1 *f1 = new TF1("f1", "1 - x^([0])", -1.,1.);
//	f1->SetParameter(0,8);  //initialize first parameter to 1.1
//	ratio_graph_SM->Fit("f1");


	TLegend *leg1 = new TLegend(0.6, 0.5, 0.9, 0.7);
//	leg1->SetFillColor(5);
	leg1->AddEntry(ratio_graph_SM,"SM","L");
	leg1->AddEntry(ratio_graph_Zprime,"E_{6}","L");
	leg1->AddEntry(ratio_graph_KK,"KK","L");
//	leg1->AddEntry(fSM,"SM_fit","L");
//	leg1->AddEntry(fE6,"E6_fit","L");
//	leg1->AddEntry(fKK,"KK_fit","L");
	leg1->Draw();

	
	string epsfile = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"efficiencies_cosTheta.eps");
	c1->Print(graph.c_str(),"root");
	c1->Update();
	c1->Print(epsfile.c_str());
	graphFile->Write();

	

	//--------------------
	cout << "\n" << "done..." << endl;
}

