/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;

using namespace Moses1;

int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;
	
	TStyle* plotstyle = (TStyle*)gROOT->GetStyle("Plain");
	plotstyle->cd();

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();


	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)

	double mStar = physPrm.mStar;
	cout << "choose: m^*=";
	cin >> mStar;
	physPrm.mStar = mStar;

	bool bDoCuts = false;
        int nDoCuts = 0;
        cout << "select cuts: without(0) or with(1)...";
        cin >> nDoCuts;
        if (nDoCuts==1) {bDoCuts = true;}

	string sWOcuts = "";
        if (bDoCuts) {
                sWOcuts = "_withCuts";
        }
        else {
                sWOcuts = "_noCuts";
        }


	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS}; // glu, gamma, Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here


	// create relevant classes instances
	couplings cplng; // set the class for couplings call
	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	lhapdf lha; // set the class for lhapdf call
	newZresonance zPrime("chi");
	bool doSMcouplings = true;
	//--------------------------------------------------------------------//


	// for the runs
	dcomplex fullAmp_SM (0,0);
	dcomplex fullAmp_KK (0,0);
	dcomplex fullAmp_Zprime (0,0);
	int nMax_ysegments = 100; // for the integration on the rapidity
	int nKKmodes = 100;
	double differentialSquareAmp_SM = 0;
	double differentialSquareAmp_KK = 0;
	double differentialSquareAmp_Zprime = 0;
	double squareAmp_SM = 0;
	double squareAmp_KK = 0;
	double squareAmp_Zprime = 0;
	double totalwidth_ZKK = 0;
	double totalwidth_Z0 = 0;
	double totalwidth_Zprime = 0;
	double totalwidth_gammaKK = 0;
	double qqbarRapidity_shat = 0; // this is the rapidity integral factor (single integral)
	double qqbarRapidity_shat_nocuts = 0;
	customParticle outFermion = CUSTOMfermions[7]; // propagator will decay into a muon muon_bar pair
	// the integral over all decay angles
	// [1] theta: distributing like ~ (1+/-cos(theta))^2
	// [2] phi: distributing uniformly
	double angularIntegral = 2*physPrm.pi * 8/3;
	

	double mHat_min = 20/*10.*/;
	double mHat_max = 6000.;
	double dmHat = 1.;
	double mHat = mHat_min;
	double s_hat = mHat_min*mHat_min;
	Int_t nentries = (Int_t)((mHat_max-mHat_min)/dmHat);
	cout << "nentries=" << nentries << endl;

	// initialize lha (lhapdf instance)
	string pdfSet = "MSTW2008lo90cl.LHgrid";
	//string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name
	
	// alocate arrays for the graph:
	Double_t *Sqrt_s_hat = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_SM = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_KK = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_Zprime = (Double_t *)malloc(nentries*sizeof(Double_t));

	string dirName = "/";
	
	string treeFilePath = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_theoryTree"+sWOcuts+".root");
	TFile *treeFile = new TFile(treeFilePath.c_str(),"RECREATE");
	string treeName = "theoryTree";
	TTree *theoryTree = new TTree(treeName.c_str(),treeName.c_str());

	theoryTree->Branch("mHat",&mHat,"mHat/D");
	theoryTree->Branch("dsigmadmHat_SM",&squareAmp_SM,"squareAmp_SM/D");
	theoryTree->Branch("dsigmadmHat_Zprime",&squareAmp_Zprime,"squareAmp_Zprime/D");
	theoryTree->Branch("dsigmadmHat_KK",&squareAmp_KK,"squareAmp_KK/D");
	

	cout << "\n\nRun over s_hat ([" << mHat_min << "->" << mHat_max << "]GeV)^2:" << endl;
	for (Int_t n=0 ; n<nentries ; n++) { // loop over s_hat
	
		squareAmp_SM = 0; // before each new s_hat it needs to be initialized to '0'
		squareAmp_KK = 0;
		squareAmp_Zprime = 0;
		totalwidth_Z0 = 0; // before each new s_hat it needs to be initialized to '0'
		totalwidth_Zprime = 0;

		// calculate the total width of Z/gamma* for each s_hat:
		// calculate the partial width to this pair of ffbar and summ it to get the total width
		for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
			totalwidth_Z0     += wdt.getpartialWidth(0, s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm); // Z0 total width
			//totalwidth_Zprime += zPrime.getZprimeWidth(doSMcouplings, CUSTOMfermions[i]);
			totalwidth_Zprime += wdt.getpartialWidth(s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm, mStar);
			if (CUSTOMfermions[i].pdt_prt.id == PDTTOP && n%1000==0)
			{
//				cout << "Z'->ttbar partial width(mHat=" << sqrt(s_hat) << ") = " << wdt.getpartialWidth(s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm, mStar) << endl;
			}
		}
		if(n%1000==0)
		{
//			cout << "Z'->FFbar total width(mHat=" << sqrt(s_hat) << ") = " << totalwidth_Zprime << endl;
		}
		

		for (int q=0 ; q<5 ; q++) { // run over all incoming quarks pairs (q+qbar) except for ttbar
			
			differentialSquareAmp_SM = 0; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_KK = 0;
			differentialSquareAmp_Zprime = 0;

			for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
				for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states (muon only)

					fullAmp_SM     = dcomplex(0,0); // before each helicity state this has to be initialized to '(0,0)'
					fullAmp_KK     = dcomplex(0,0);
					fullAmp_Zprime = dcomplex(0,0);

					// ---------- SM terms ----------------------------------------------------------------------
					// photon propagator
					fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, 0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[1], physPrm);
					// Z0 propagator
					fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, totalwidth_Z0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[2], physPrm);
					// ------------------------------------------------------------------------------------------


					// ---------- KK terms ----------------------------------------------------------------------
					fullAmp_KK = fullAmp_SM;
					for (int KKmode=1 ; KKmode<=nKKmodes ; KKmode++) { // loop over all KK modes
						totalwidth_gammaKK = 0.;
						totalwidth_ZKK     = 0.;
						for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
							switch (CUSTOMfermions[i].pdt_prt.id) {
								case PDTNUE: break;
								case PDTNUMU: break;
								case PDTNUTAU: break;
								default: // gamma can not decay into neutrino pair (while Z can)
									totalwidth_gammaKK += wdt.getpartialWidth(KKmode, s_hat, PDTbosons[1],
												CUSTOMfermions[i], physPrm); // gammaKK tot width
									if (CUSTOMfermions[i].pdt_prt.id == PDTTOP && KKmode==1 && hIn==-0.5 &&  hOut==-0.5 && q==0 && n%1000==0)
									{
//										cout << "gamma^*->ttbar partial width(nKK=" << KKmode << ", mHat=" << sqrt(s_hat) << ") = " << wdt.getpartialWidth(KKmode, s_hat, PDTbosons[1],CUSTOMfermions[i], physPrm) << endl;
									}
									break;
							}
							totalwidth_ZKK    += wdt.getpartialWidth(KKmode, s_hat, PDTbosons[2],
										CUSTOMfermions[i], physPrm); // Z* total width
							if (CUSTOMfermions[i].pdt_prt.id == PDTTOP && KKmode==1 && hIn==-0.5 &&  hOut==-0.5 && q==0 && n%1000==0)
							{
//								cout << "Z^*->ttbar partial width(nKK=" << KKmode << ", mHat=" << sqrt(s_hat) << ") = " << wdt.getpartialWidth(KKmode, s_hat, PDTbosons[2],CUSTOMfermions[i], physPrm) << endl;
							}
						}
						if(KKmode==1 && hIn==-0.5 &&  hOut==-0.5 && q==0 && n%1000==0)
						{
//							cout << "gamma^*->FFbar total width(nKK=" << KKmode << ", mHat=" << sqrt(s_hat) << ") = " << totalwidth_gammaKK << endl;
//							cout << "Z^*->FFbar total width(nKK=" << KKmode << ", mHat=" << sqrt(s_hat) << ") = " << totalwidth_ZKK << endl;
						}
						
						// photon* propagator
						fullAmp_KK += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_gammaKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[1], physPrm);

						// Z* propagator
						fullAmp_KK += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_ZKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[2], physPrm);
					}
					// ------------------------------------------------------------------------------------------


					// ---------- Z' terms ----------------------------------------------------------------------
					fullAmp_Zprime = zPrime.getSquaredAmplitude(hIn,hOut,s_hat,CUSTOMfermions[q],outFermion,
										totalwidth_Z0,totalwidth_Zprime,doSMcouplings);
					// ------------------------------------------------------------------------------------------


					// get the differential square amplitude: |fullAmp|^2
					differentialSquareAmp_SM     += (s_hat*s_hat) * (1./4.) * real( fullAmp_SM*conj(fullAmp_SM) );
					differentialSquareAmp_KK     += (s_hat*s_hat) * (1./4.) * real( fullAmp_KK*conj(fullAmp_KK) );
					differentialSquareAmp_Zprime += (s_hat*s_hat) * (1./4.) * real( fullAmp_Zprime*conj(fullAmp_Zprime) );
				} // end hOut
			} // end hIn
		

			if (!bDoCuts) {
                                qqbarRapidity_shat = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha);
				//if (qqbarRapidity_shat<0.) cout << "#################### qqbarRapidity_shat without cuts= " << qqbarRapidity_shat << endl;
                                // the qqbar rapidity integral factor without cuts
                        }
                        else {
                                qqbarRapidity_shat = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha,cuts);
				//if (qqbarRapidity_shat<0.) cout << "#################### qqbarRapidity_shat with cuts= " << qqbarRapidity_shat << endl;
                                // the qqbar rapidity integral factor with cuts
                        }
			qqbarRapidity_shat_nocuts = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha); // the qqbar rapidity integral factor
			// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
			// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
			//     and here only 3 are valid sice the propogator is colorless
			// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams

			if (!bDoCuts) {
                                // no cuts
                                differentialSquareAmp_SM     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
                                differentialSquareAmp_KK     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
                                differentialSquareAmp_Zprime *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
                        }
                        else {
                                // with cuts
                                // qqbarRapidity_shat already includes the modified integral over theta (because it depend on rapidity)
                                differentialSquareAmp_SM     *= 2*(qqbarRapidity_shat/(8./3.))/CUSTOMfermions[q].Nc;
                                differentialSquareAmp_KK     *= 2*(qqbarRapidity_shat/(8./3.))/CUSTOMfermions[q].Nc;
                                differentialSquareAmp_Zprime *= 2*(qqbarRapidity_shat/(8./3.))/CUSTOMfermions[q].Nc;
                        }

			squareAmp_SM     += differentialSquareAmp_SM;  // only then add the differential square amplitude to the full one
			squareAmp_KK     += differentialSquareAmp_KK;
			squareAmp_Zprime += differentialSquareAmp_Zprime;

		} // end q


		// multiply by the coefficients appearing before the squared helicity amplitude
		squareAmp_SM     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
		squareAmp_KK     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
		squareAmp_Zprime *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
			// [squareAmp] = [1/GeV^4]
		squareAmp_SM     *= 2*sqrt(s_hat); // dSigma(s_hat) -> dSigma(sqrt(s_hat)) Jacobian of the transformation
		squareAmp_KK     *= 2*sqrt(s_hat);
		squareAmp_Zprime *= 2*sqrt(s_hat);
			// [squareAmp] = [1/GeV^3]
		squareAmp_SM     *= physPrm.GeV2nbarn; // to convert the units to nb/GeV (originally was nb/GeV^2 but here it is nb/GeV)
		squareAmp_KK     *= physPrm.GeV2nbarn;
		squareAmp_Zprime *= physPrm.GeV2nbarn;
			// [squareAmp] = [nb/GeV]
		// see more details about the conversion using the method "prm.printUnits()" in the end of the loop. 

		// fill arrays of graphs:
		Sqare_amp_SM[n]     = squareAmp_SM;
		Sqare_amp_KK[n]     = squareAmp_KK;
		Sqare_amp_Zprime[n] = squareAmp_Zprime;
		Sqrt_s_hat[n] = mHat;

		theoryTree->Fill();

		double ratio = qqbarRapidity_shat/qqbarRapidity_shat_nocuts;
		if(bDoCuts) {ratio /= (8./3.);}
		cout << "sqrt(s_hat)=" << mHat  << "                     rapidity factor:  cuts/nocuts=" << ratio << "\r" << flush;
		mHat += dmHat;
		s_hat = mHat*mHat;

	} // end sMat
	prm.printUnits(); // just a reminder about units...
	theoryTree->Print();
	treeFile->Write();

/*
	//--------------------------------------------------------------------------
	Double_t dx = 0.0001;
	Double_t xMin=-1., xMax=+1.;
	Int_t nEntries = (Int_t)((xMax-xMin)/dx);
	Double_t *x = (Double_t *)malloc(nEntries*sizeof(Double_t));
        Double_t *fx = (Double_t *)malloc(nEntries*sizeof(Double_t));

	for (Int_t n=0 ; n<nEntries ; n++) {
		x[n]  = xMin + (Double_t)n*dx; 
		fx[n] = x[n]*x[n] + 1.;
	}

	double integral_fx = commonOps.simpsonIntegration(fx, x, nEntries, 0., 0.);
	cout << "\nintegral_fx = " << integral_fx << "\n" << endl;
	//--------------------------------------------------------------------------
*/


	Int_t f1=0, f2=0;
	for(Int_t m=0 ; m<nentries ; m++)
	{
		if (Sqrt_s_hat[m]==1000.) {f1=m;}
		if (Sqrt_s_hat[m]==6000.) {f2=m;}
	}
	if (f2==0) f2 = nentries-1;
	cout << "Sqrt_s_hat[f1=" << f1 << "]=" << Sqrt_s_hat[f1] << endl;
	cout << "Sqrt_s_hat[f2=" << f2 << "]=" << Sqrt_s_hat[f2] << endl;
	double nb2fb = 1.e6;
	double luminosity = 100.; //fb^-1
	double Nsm = commonOps.simpsonIntegration(Sqare_amp_SM,     Sqrt_s_hat, nentries, f1, f2)*nb2fb*luminosity;
	double Nkk = commonOps.simpsonIntegration(Sqare_amp_KK,     Sqrt_s_hat, nentries, f1, f2)*nb2fb*luminosity;
	double Ne6 = commonOps.simpsonIntegration(Sqare_amp_Zprime, Sqrt_s_hat, nentries, f1, f2)*nb2fb*luminosity;
	cout << "\nNsm=" << Nsm << ", Nkk=" << Nkk << ", Ne6=" << Ne6 << "\n" << endl;


/*
	//-----------------------------------------------------------------------------------------
        string treeFilePath_tmp = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"Afb_evg.root");
        TFile *treeFile_tmp = new TFile(treeFilePath_tmp.c_str(),"RECREATE");
        string treeName_tmp = "AfbTheoryTree";
        TTree *AfbTheoryTree = new TTree(treeName_tmp.c_str(),treeName.c_str());

        Double_t mH;
        Double_t Afb_KK;
        Double_t Afb_E6;
        Double_t Afb_SM;

        AfbTheoryTree->Branch("mH",&mH,"mH/D");
        AfbTheoryTree->Branch("Afb_SM",&Afb_SM,"Afb_SM/D");
        AfbTheoryTree->Branch("Afb_E6",&Afb_E6,"Afb_E6/D");
        AfbTheoryTree->Branch("Afb_KK",&Afb_KK,"Afb_KK/D");

	// create the graphs and register them to root files
	// read from TAU dat file
	ifstream inputFile;
	string Afb_evg = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"Afb_evg.TAU");
	inputFile.open(Afb_evg.c_str());

	Int_t n = 0;
	while(!inputFile.eof()) {
		inputFile >> mH;
		inputFile >> Afb_KK;
		inputFile >> Afb_E6;
		inputFile >> Afb_SM;
		AfbTheoryTree->Fill();
		cout << "mH=" << mH << "\r" << flush;
		if (!inputFile.eof()) {n++;}
	}
	cout << "n=" << n << endl;
	inputFile.close();
	AfbTheoryTree->Print();
	treeFile_tmp->Write();
	//----------------------------------------------------------------------------------
*/
	string inputTreeFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"Afb_evg.root");
	TFile *inputTreeFile     = new TFile(inputTreeFileName.c_str());
	string inputTreeName     = "AfbTheoryTree";
	TTree *AfbTree           = (TTree*)inputTreeFile->Get(inputTreeName.c_str());

	Double_t mH;
	Double_t Afb_KK;
	Double_t Afb_E6;
	Double_t Afb_SM;

	AfbTree->SetBranchAddress("mH",&mH);
	AfbTree->SetBranchAddress("Afb_KK",&Afb_KK);
	AfbTree->SetBranchAddress("Afb_E6",&Afb_E6);
	AfbTree->SetBranchAddress("Afb_SM",&Afb_SM);

	
	Double_t mHat_Afb_min = 20./*300.*/;
	Double_t mHat_Afb_max = 6000./*5500.*/;
	Double_t dmHat_Afb    = 1.;

	Int_t nentriesAfb = (Int_t)((mHat_Afb_max - mHat_Afb_min)/dmHat_Afb);

	Double_t *mH_array     = (Double_t *)malloc(nentriesAfb*sizeof(Double_t));
	Double_t *Afb_KK_array = (Double_t *)malloc(nentriesAfb*sizeof(Double_t));
	Double_t *Afb_E6_array = (Double_t *)malloc(nentriesAfb*sizeof(Double_t));
	Double_t *Afb_SM_array = (Double_t *)malloc(nentriesAfb*sizeof(Double_t));
	
	Int_t treeNentries = (Int_t)AfbTree->GetEntries();
	cout << "treeNentries = " << treeNentries << endl;
	cout << "nentriesAfb  = " << nentriesAfb << endl;
	Int_t n=0;
	for (Int_t i=0 ; i<treeNentries ; i++) {
		AfbTree->GetEntry(i);
		if (mH>=mHat_Afb_min && mH<=mHat_Afb_max && n<nentriesAfb) {
			mH_array[n]     = mH;
			Afb_KK_array[n] = Afb_KK;
			Afb_E6_array[n] = Afb_E6;
			Afb_SM_array[n] = Afb_SM;
			n++;
		}
	}


        // calculate the integral of the theoretical line (simpson integration method)
        Double_t integrand_SM = 0;
        Double_t integrand_KK = 0;
        Double_t integrand_E6 = 0;

        Double_t sigmaTot_SM = 0;
        Double_t sigmaTot_KK = 0;
        Double_t sigmaTot_E6 = 0;

        Double_t sigmaTot_Afb_SM = 0;
        Double_t sigmaTot_Afb_KK = 0;
        Double_t sigmaTot_Afb_E6 = 0;

	
	// find the index of mKKpeak_min in the Sqrt_s_hat and mH_array arrays

	Double_t mKKpeak_min = 2000.;
	Double_t mKKpeak_max = 5000.;
	bool stop = false;
	n=0;
	while (!stop) {
		if (Sqrt_s_hat[n] == mKKpeak_min) {stop = true;}
		n++;
	}
	cout << "\n" << "starting point: n=" << n << endl;

	stop = false;
	Int_t m=0;
	while (!stop) {
		if (mH_array[m] == mKKpeak_min) {stop = true;}
                m++;
	}
	cout << "starting point: =" << m << "\n" << endl;

	int nSegments = (int)nentriesAfb-1;
        Double_t h = dmHat_Afb/3;
	int i=0;
        while (i<=nSegments && (i+m)<nentriesAfb && (i+n)<nentries) {
                // calculate the integrand:
                integrand_SM = Sqare_amp_SM[n+i];
                integrand_KK = Sqare_amp_KK[n+i];
                integrand_E6 = Sqare_amp_Zprime[n+i];
                // simpson:
                if (i==0 || i==nSegments) {
                       sigmaTot_SM += integrand_SM;
                        sigmaTot_KK += integrand_KK;
                        sigmaTot_E6 += integrand_E6;
			sigmaTot_Afb_SM += integrand_SM * Afb_SM_array[m+i];
			sigmaTot_Afb_KK += integrand_KK * Afb_KK_array[m+i];
			sigmaTot_Afb_E6 += integrand_E6 * Afb_E6_array[m+i];
                }
                else if (i%2==0 && i!=0 && i!=nSegments) {
                        sigmaTot_SM += 2*integrand_SM;
                        sigmaTot_KK += 2*integrand_KK;
                        sigmaTot_E6 += 2*integrand_E6;
			sigmaTot_Afb_SM += 2.*integrand_SM * Afb_SM_array[m+i];
			sigmaTot_Afb_KK += 2.*integrand_KK * Afb_KK_array[m+i];
			sigmaTot_Afb_E6 += 2.*integrand_E6 * Afb_E6_array[m+i];
                }
		else {
                        sigmaTot_SM += 4*integrand_SM;
                        sigmaTot_KK += 4*integrand_KK;
                        sigmaTot_E6 += 4*integrand_E6;
			sigmaTot_Afb_SM += 4.*integrand_SM * Afb_SM_array[m+i];
			sigmaTot_Afb_KK += 4.*integrand_KK * Afb_KK_array[m+i];
			sigmaTot_Afb_E6 += 4.*integrand_E6 * Afb_E6_array[m+i];
                }
                // propagate i:
                i++;
        }
        sigmaTot_SM *= h; // [nb]
        sigmaTot_KK *= h; // [nb]
        sigmaTot_E6 *= h; // [nb]

        sigmaTot_Afb_SM *= h; // [nb]
        sigmaTot_Afb_KK *= h; // [nb]
        sigmaTot_Afb_E6 *= h; // [nb]


	cout << "Afb(SM)[" << mKKpeak_min << "<mHat<" << mKKpeak_max << "] = " << sigmaTot_Afb_SM/sigmaTot_SM << endl;
	cout << "Afb(E6)[" << mKKpeak_min << "<mHat<" << mKKpeak_max << "] = " << sigmaTot_Afb_E6/sigmaTot_E6 << endl;
	cout << "Afb(KK)[" << mKKpeak_min << "<mHat<" << mKKpeak_max << "] = " << sigmaTot_Afb_KK/sigmaTot_KK << endl;
	

	TGraph *Afb_KK_graph = new TGraph (nentriesAfb, mH_array, Afb_KK_array);
	TGraph *Afb_E6_graph = new TGraph (nentriesAfb, mH_array, Afb_E6_array);
	TGraph *Afb_SM_graph = new TGraph (nentriesAfb, mH_array, Afb_SM_array);
 	


	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_theory"+sWOcuts+".root");
        TFile *rootFile = new TFile(rootFileName.c_str(),"RECREATE");

	string epsFileAfb = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_Afb_theory.eps");


	TDirectory* dir_allHistograms = rootFile->mkdir("allHistograms");
        dir_allHistograms->cd();

	TDirectory* dir_Afb = rootFile->mkdir("Afb");
        dir_Afb->cd();
	TCanvas *canv_Afb = new TCanvas("Afb","Afb",0,0,1400,800);

	Afb_SM_graph->SetMaximum(0.5/*1.25*Afb_SM_graph->GetMaximum()*/);
	Afb_SM_graph->SetMinimum(-0.5/*1.25*Afb_E6_graph->GetMinimum()*/);

	canv_Afb->Draw();
        canv_Afb->cd();
	Afb_SM_graph->SetTitle("");
	Afb_SM_graph->SetLineColor(kBlue);
	Afb_SM_graph->SetLineStyle(5);
	Afb_SM_graph->GetXaxis()->SetTitle("#sqrt{#hat s} [GeV]");
	Afb_SM_graph->GetYaxis()->SetTitle("A_{fb}^{#beta}");
	Afb_SM_graph->Draw("AC");

	Afb_E6_graph->SetLineStyle(3);
	Afb_E6_graph->SetLineColor(kBlack);
	Afb_E6_graph->Draw("C");

	Afb_KK_graph->SetLineStyle(1);
	Afb_KK_graph->SetLineColor(kRed);
	Afb_KK_graph->Draw("C");

	TLegend *leg_mHat_Afb = new TLegend(0.6, 0.3, 0.98, 0.5);
	leg_mHat_Afb->AddEntry(Afb_SM_graph,"SM","L");
	leg_mHat_Afb->AddEntry(Afb_E6_graph,"Z'_{SM}","L");
	leg_mHat_Afb->AddEntry(Afb_KK_graph,"KK","L");
	leg_mHat_Afb->Draw();

	canv_Afb->Update();
	canv_Afb->Write();

	

	string epsFilemHat = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_invMass_theory"+sWOcuts+".eps");

	TDirectory* dir_mHat = rootFile->mkdir("mHat");
        dir_mHat->cd();
	TCanvas *canv_mHat = new TCanvas("canv_mHat","canv_mHat",0,0,1400,800);
	canv_mHat->SetLogy();
	TGraph *crossSection_SM = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_SM);
	TGraph *crossSection_KK = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_KK);
	TGraph *crossSection_Zprime = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_Zprime);

	canv_mHat->Draw();
	canv_mHat->cd();
	crossSection_SM->SetMinimum(1.e-14);
	crossSection_SM->SetTitle("");
	crossSection_SM->SetLineColor(kBlue);
	crossSection_SM->SetLineStyle(5);
	crossSection_SM->GetXaxis()->SetTitle("#sqrt{#hat s} [GeV]");
	crossSection_SM->GetYaxis()->SetTitle("#frac{d#sigma}{d#sqrt{#hat s}} [#frac{nb}{GeV}]");
	crossSection_SM->Draw("AC");
	
	crossSection_Zprime->SetLineStyle(3);
	crossSection_Zprime->SetLineColor(kBlack);
	crossSection_Zprime->Draw("C");

	crossSection_KK->SetLineStyle(1);
	crossSection_KK->SetLineColor(kRed);
	crossSection_KK->Draw("C");

	TLegend *leg_mHat = new TLegend(0.6, 0.7, 0.98, 0.9);
	leg_mHat->AddEntry(crossSection_SM,"SM","L");
	leg_mHat->AddEntry(crossSection_Zprime,"Z'_{SM}","L");
	leg_mHat->AddEntry(crossSection_KK,"KK","L");
	leg_mHat->Draw();

	canv_mHat->Update();
	canv_mHat->Write();



	rootFile->Write();

	canv_Afb->Print(epsFileAfb.c_str());
	canv_mHat->Print(epsFilemHat.c_str());

        canv_mHat->WaitPrimitive();
	canv_Afb->WaitPrimitive();

	return 0;
}

