#ifndef PYTHIAPLUGIN_h
#define PYTHIAPLUGIN_h
/*********************************************/
/* INTERFACE CLASSES TO Pythia8              */
/* feed pythia with a completely new process */
/*********************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------


using namespace std;
using namespace Pythia8; 

namespace Moses1 {

// A derived class for a completely new, user defined (from scratch) process
class eebar2mumubar : public Sigma2Process {
	public:
		// Constructor.
		eebar2mumubar(int idIn) : m_idNew(idIn) {}

		// initialize the process:
		void initProc();

		// Calculate flavour-independent parts of cross section.
		virtual void sigmaKin();

		// Evaluate sigmaHat(sHat). 
		virtual double sigmaHat();

		// Select flavour, colour and anticolour.
		virtual void setIdColAcol();

		virtual string	name()		const {return "e+e-->mu+mu-";}
		virtual int	code()		const {return 100001;}
		virtual string	inFlux()	const {return "ffbarSame";}
		virtual bool	isSChannel() 	const {return true;}
		virtual bool    hasIntermediate() const {return true;}
		virtual int     idIntermediate()  const {return 10022;}
		virtual int	id3Mass()	const {return m_idNew;}
		virtual int	id4Mass()	const {return m_idNew;}
		virtual bool	convert2mb()	const {return true;}


	private:
		// Store flavour-specific process information and standard prefactor.
		double m_sigma;
		int m_idNew;
		bool m_isPhysical;
		double m_mr, m_betaf, m_cosThe;
		double m_thetaDistribution;
};


}
#endif
