/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/
#include "pythiaSemiExternalPlugin.h"

using namespace std;
using namespace Moses1;



int main() {
	cout << "Holy Moses..." << endl;
	
	
	cout << "\n\n" << "-------------------Set a user defined process run with Pythia8-------------------" << endl;
	// Number of events to generate and to list. Max number of errors.
	// Warning: generation of complete events is much slower than if you use
	// PartonLevel:all = off to only get cross sections, so adjust nEvent.
	
	// Pythia generator.
	Pythia pythia;
	int outFermionID = 13;

	// A class to generate the process from external matrix element. 
	SigmaProcess* myProcess = new PYTHIA8gmZ2mumubar(outFermionID,outFermionID);
	
	// Hand pointer to Pythia.
	pythia.setSigmaPtr(myProcess);
	cout << "Hand pointer to Pythia" << endl;

	//pythia.readString("WeakZ0:gmZmode = 1"); // only photon propagator (can be usefull for tests)
	//pythia.readString("WeakZ0:gmZmode = 2"); // only Z0 propagator (can be usefull for tests)

	// Chosen new PDF set; LHAPDF file name conventions.
	//string pdfSet = "cteq61.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "MRST2006nnlo.LHgrid";
	pythia.readString("PDF:useLHAPDF = on");
	pythia.readString("PDF:LHAPDFset = " + pdfSet);
	// Allow extrapolation of PDF's beyond x and Q2 boundaries, at own risk.
	// Default behaviour is to freeze PDF's at boundaries.
	pythia.readString("PDF:extrapolateLHAPDF = on");
	// Need to change pT0Ref depending on choice of PDF.
	// One possibility: retune to same <n_charged>.
	//pythia.readString("MultipleInteractions:pT0Ref = 5");


	// phase space cuts
	pythia.readString("PhaseSpace:mHatMin = 20");
	//pythia.readString("PhaseSpace:mHatMax = 200");


	// Optionally only compare cross sections.
	string partonLevelSwitch = "n";
	cout << "switch off PartonLevel:all ? (y/n)......";
	cin >> partonLevelSwitch;
	if (partonLevelSwitch == "y") {
		pythia.readString("PartonLevel:all = off");
		//pythia.readString("PartonLevel:ISR = off");
	}


	// Initialization for LHC.
	pythia.init(2212, 2212, 14000.);
	cout << "Initialization for LHC" << endl;

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();

	commonOperations commonOps;
	//string filePath = commonOps.checkANDsetFilepath("MOSESDATA", "/pythiaSemiExternal_muons.dat");
	//ofstream muFile;
	//muFile.open(filePath.c_str());

	string dirName = "/";

	string filePath3 = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"pythia_semiexternal_muons.root");
	string treeName = "testTree";
	TFile *treeFile = new TFile(filePath3.c_str(),"recreate");
	TTree *tree = new TTree(treeName.c_str(),treeName.c_str());
	int eventNumber;
	double mHat, thetaHat, cosThetaHat;
	tree->Branch("eventNumber",&eventNumber,"eventNumber/I");
	tree->Branch("mHat",&mHat,"mHat/D");
	tree->Branch("thetaHat",&thetaHat,"thetaHat/D");
	

	int nEvents = 200;

	cout << "\n\n" << "---------------------------Start the run--------------------------" << endl;
	cout << "\n\nenter the number of MUON-PAIRS to loop over (integer>0, default is 200): ";
	cin >> nEvents;
	cout << "start running and booking events to files..." << endl;

	Vec4 p1,p2,pSum,p, pq;
	int idL1=0, idL2=0;

	// run
	int mod = (int)nEvents/10;
	int evt=1;
	while (evt <= nEvents) {
		pythia.next();
	
		eventNumber = evt;
		mHat = pythia.info.mHat();
		//---- in a 2->1->2 process thetaHat is not given in by pythia.info.thetaHat() unlike mHat.------------------
		// therefore we have to calculate it explicitly from the process record (not the event record)
		for (int i=0 ; i<((int)pythia.process.size()-1) ; i++) {
			if (abs(pythia.process[i].id()) == outFermionID && abs(pythia.process[i+1].id()) == outFermionID) {
				idL1 = pythia.process[i].id();
				p1.e(pythia.process[i].e());
				p1.px(pythia.process[i].px());
				p1.py(pythia.process[i].py());
				p1.pz(pythia.process[i].pz());
				idL2 = pythia.process[i+1].id();
				p2.e(pythia.process[i+1].e());
				p2.px(pythia.process[i+1].px());
				p2.py(pythia.process[i+1].py());
				p2.pz(pythia.process[i+1].pz());
			}
		}
		pSum = p1 + p2; // proportional to the boot
		if (idL1 > 0) {
			p1.bstback(pSum); // boost back with beta=-veq{pSum}/pSum^0  (beta = -p/E)
			p = p1;
		}
		else if (idL2 > 0) {
			p2.bstback(pSum); // boost back with beta=-veq{pSum}/pSum^0  (beta = -p/E)
			p = p2;
		}
		// calculate cos(theta*) relative to the boost
		cosThetaHat =   ( p.px()*pSum.px() + p.py()*pSum.py() + p.pz()*pSum.pz() ) /
				(
					sqrt(p.px()*p.px() + p.py()*p.py() + p.pz()*p.pz()) *
					sqrt(pSum.px()*pSum.px() + pSum.py()*pSum.py() + pSum.pz()*pSum.pz())
				); // cosTheta between the vectors wrt the boost direction.
		//cosThetaHat = p.pz()*pq.pz()/(sqrt(p.px()*p.px() + p.py()*p.py() + p.pz()*p.pz())*abs(pq.pz()));
		//-----------------------------------------------------------------------------------------------------------
		thetaHat = acos(cosThetaHat);
		//thetaHat = pythia.info.thetaHat();
		tree->Fill();

		//muFile << evt << "\t" << pythia.info.mHat() << "\n";
		if (evt%mod==0) {cout << "event: " << evt << endl;}
		//if (evt%mod==0) {pythia.process.list();}
		evt++;
	}
	tree->Print();
	treeFile->Write();	
	//muFile.close();
	pythia.statistics();

	// Done.
	return 0;
}

