/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Moses1;

int main() {
	cout << "Holy Moses..." << endl;
	

	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct


	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS}; // glu, gamma, Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here


	// create relevant classes instances
	couplings cplng; // set the class for couplings call
	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	lhapdf lha; // set the class for lhapdf call
	commonOperations commonOps; // set the class for commonOperations call
	//--------------------------------------------------------------------//

	// for the runs
	dcomplex gplus (0,0);
	dcomplex gminus (0,0);
	dcomplex fullAmp (0,0);
	int nMax_ysegments = 100; // for the integration on the rapidity
	double differentialSquareAmp = 0;
	double squareAmp = 0;
	double totalwidth = 0;
	double qqbarRapidity_shat = 0; // this is the rapidity integral factor (single integral)
	customParticle outFermion = CUSTOMfermions[7]; // propagator will decay into a muon muon_bar pair
	// the integral over all decay angles
	// [1] theta: distributing like ~ (1+/-cos(theta))^2
	// [2] phi: distributing uniformly
	double angularIntegral = 2*physPrm.pi * 8/3;
	
	double mHat_min = 20.;
	double mHat_max = 200.;
	double dmHat = 1.;
	double mHat = mHat_min;
	double s_hat = mHat_min*mHat_min;
	Int_t nentries = (Int_t)((mHat_max-mHat_min)/dmHat) + (Int_t)dmHat;
	cout << "nentries=" << nentries << endl;
	

	// alocate arrays for the graph:
	Double_t *Sqrt_s_hat = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Z0width = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *rapidityIntegral_d = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *rapidityIntegral_u = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *rapidityIntegral_s = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *rapidityIntegral_c = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *rapidityIntegral_b = (Double_t *)malloc(nentries*sizeof(Double_t));

	Double_t *rapidityIntegral_largeN = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *rapidityIntegral_mediumN = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *rapidityIntegral_smallN = (Double_t *)malloc(nentries*sizeof(Double_t));


	// initialize lha (lhapdf instance)
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	//string pdfSet = "MRST2006nnlo.LHgrid";
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name


/*
	// ------------------- check the simpson integral method:
	// sum_f[integrate{x*pdf(x,Q,f)dx}] =? 1
	int nMax_xsegments = 10000;
	double Q = 20;
	double test = rap.simpsonIntegralTest(nMax_xsegments, Q, lha);
	cout << "for Q=" << Q << " " << rap.nActualSegments << " segments" << ", sum_f[integrate{x*pdf(x,Q,f)dx}]=?1:....." << test << "  [" << (1-test)*100 << "%]" << "\n" << endl;
	Q = 91.17;
	test = rap.simpsonIntegralTest(nMax_xsegments, Q, lha);
	cout << "for Q=" << Q << " " << rap.nActualSegments << " segments" << ", sum_f[integrate{x*pdf(x,Q,f)dx}]=?1:....." << test << "  [" << (1-test)*100 << "%]" << "\n" << endl;
	Q = 120;
	test = rap.simpsonIntegralTest(nMax_xsegments, Q, lha);
	cout << "for Q=" << Q << " " << rap.nActualSegments << " segments" << ", sum_f[integrate{x*pdf(x,Q,f)dx}]=?1:....." << test << "  [" << (1-test)*100 << "%]" << "\n" << endl;
	Q = 180;
	test = rap.simpsonIntegralTest(nMax_xsegments, Q, lha);
	cout << "for Q=" << Q << " " << rap.nActualSegments << " segments" << ", sum_f[integrate{x*pdf(x,Q,f)dx}]=?1:....." << test << "  [" << (1-test)*100 << "%]" << "\n" << endl;
	// ------------------- \
	*/

	// write to dat file
	ofstream ouputFile;
	string resultsFile = commonOps.checkANDsetFilepath("MOSESDATA","/sigma.SM");
	ouputFile.open(resultsFile.c_str());

	string filePath = commonOps.checkANDsetFilepath("MOSESDATA", "/dsigmadmHat_SM_theory.root");
	string treeName = "testTree";
	TFile *treeFile = new TFile(filePath.c_str(),"recreate");
	TTree *tree = new TTree(treeName.c_str(),treeName.c_str());
	double mHat_SM, dsigmadmHat_SM;
	tree->Branch("mHat_SM",&mHat_SM,"mHat_SM/D");
	tree->Branch("dsigmadmHat_SM",&dsigmadmHat_SM,"dsigmadmHat_SM/D");


	cout << "\n\nRun over s_hat ([" << mHat_min << "->" << mHat_max << "]GeV)^2:" << endl;
	for (Int_t n=0 ; n<nentries ; n++) { // loop over s_hat
	//while (mHat <= mHat_max) {
		
		squareAmp = 0; // before each new s_hat it needs to be initialized to '0'
		totalwidth = 0; // before each new s_hat it needs to be initialized to '0'

		// calculate the total width of Z0 for each s_hat:
		for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
			// calculate the partial width to this pair of f fbar
			totalwidth += wdt.getpartialWidth(0, s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm);
			// the first argument '0' stands for SM calculations (the 0th mode of Z boson)
		}
		
		for (int q=0 ; q<5 ; q++) { // run over all incoming quarks pairs (q+qbar) except for ttbar
			
			differentialSquareAmp = 0; // before each new q it needs to be initialized to '0'

			for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
				for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states (muon only)

					fullAmp = dcomplex(0,0); // before each helicity state this has to be initialized to '(0,0)'
				
					fullAmp += hAmp.gethelicityAmplitude(s_hat, 0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[1], physPrm); // photon propagator
					
					fullAmp += hAmp.gethelicityAmplitude(s_hat, totalwidth, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[2], physPrm); // Z0 propagator
					
					// get the differential square amplitude: |fullAmp|^2
					differentialSquareAmp += (s_hat*s_hat) * (1./4.) * real( fullAmp*conj(fullAmp) );

				} // end hOut
			} // end hIn
			
			qqbarRapidity_shat = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha); // the qqbar rapidity integral factor
			if (q==PDTUP) {
				rapidityIntegral_largeN[n] = rap.simpsonIntegral(10000,s_hat,CUSTOMfermions[q],runPrm,lha);
				rapidityIntegral_mediumN[n] = rap.simpsonIntegral(100,s_hat,CUSTOMfermions[q],runPrm,lha);
				rapidityIntegral_smallN[n] = rap.simpsonIntegral(3,s_hat,CUSTOMfermions[q],runPrm,lha);
			}

			switch(q) { // fill the arrays of the rapidity factors for each partons pair (q+qbar)
				case PDTDWN: rapidityIntegral_d[n] = qqbarRapidity_shat; break;
				case PDTUP:  rapidityIntegral_u[n] = qqbarRapidity_shat; break;
				case PDTSTR: rapidityIntegral_s[n] = qqbarRapidity_shat; break;
				case PDTCHM: rapidityIntegral_c[n] = qqbarRapidity_shat; break;
				case PDTBOT: rapidityIntegral_b[n] = qqbarRapidity_shat; break;
				//case PDTTOP: rapidityIntegral_t[n] = qqbarRapidity_shat; break;
			}
			
			// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
			// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
			//     and here only 3 are valid sice the propogator is colorless
			// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams
			differentialSquareAmp *= 2*qqbarRapidity_shat/CUSTOMfermions[q].Nc;

			squareAmp += differentialSquareAmp;  // only then add the differential square amplitude to the full one
			
		} // end q
		
		// multiply by the coefficients appearing before the squared helicity amplitude
		squareAmp *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
			// [squareAmp] = [1/GeV^4]
		squareAmp *= 2*sqrt(s_hat); // dSigma(s_hat) -> dSigma(sqrt(s_hat)) Jacobian of the transformation
			// [squareAmp] = [1/GeV^3]
		squareAmp *= physPrm.GeV2nbarn; // to convert the units to nb/GeV (originally was nb/GeV^2 but here it is nb/GeV)
			// [squareAmp] = [nb/GeV]

		// see more details about the conversion using the method "prm.printUnits()" in the end of the loop. 

		// fill arrays of graphs:
		Sqare_amp[n] = squareAmp;
		Sqrt_s_hat[n] = mHat;
		Z0width[n] = totalwidth;

		mHat_SM = mHat;
		dsigmadmHat_SM = squareAmp;
		tree->Fill();

		ouputFile << mHat << "\t\t" << squareAmp << endl;

		cout << "sqrt(s_hat)=" << mHat << "\r" << flush;
		mHat += dmHat;
		s_hat = mHat*mHat;

	} // end mHat
	tree->Print();
	treeFile->Write();
	

	prm.printUnits(); // just a reminder about units...

	cout << "\n\n" << "--------------creating graph--------------" << endl;
	// create the graphs and register them to root files
	string graph1 = commonOps.checkANDsetFilepath("MOSESDATA","/sigmaANDrapidity_qqbar2mumubar.root");
	string graph2 = commonOps.checkANDsetFilepath("MOSESDATA","/Z0width_qqbar2mumubar.root");
	string graph3 = commonOps.checkANDsetFilepath("MOSESDATA","/rapidity.root");

	TFile *graphFile1 = new TFile(graph1.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph
	TFile *graphFile2 = new TFile(graph2.c_str(),"RECREATE"); // new root file for the Z0 width graph
	TFile *graphFile3 = new TFile(graph3.c_str(),"RECREATE"); // new root file for the rapidity

	TCanvas *c1 = new TCanvas("c1","qqbar->(Z0+gamma)->mu-mu+  dSigma(s)/ds and Rapidity integrals",0,0,1400,800);
	c1->Divide(1,2);
	
	c1->cd(1);
	TPad *pad1 = (TPad*)c1->GetPad(1);
	//TVirtualPad *pad1=c1->cd(1);
	pad1->SetFillColor(37);
	pad1->SetLogy();
	pad1->SetGrid();
	TGraph *crossSection = new TGraph (nentries, Sqrt_s_hat, Sqare_amp);
	crossSection->SetTitle("qqbar->(Z0+gamma)->mu-mu+ differential cross section");
	crossSection->SetLineColor(kRed);
	crossSection->GetXaxis()->SetTitle("Sqrt(s_hat) [GeV]");
	crossSection->GetYaxis()->SetTitle("differential cross section [nb/GeV]");
	crossSection->Draw("AC");

	c1->cd(2);
	TPad *pad2 = (TPad*)c1->GetPad(2);
	//TVirtualPad *pad2=c1->cd(2);
	pad2->SetFillColor(37);
	pad2->SetLogy();
	pad2->SetGrid();
	TGraph *graph_rapidity_d = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_d);
	TGraph *graph_rapidity_u = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_u);
	TGraph *graph_rapidity_s = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_s);
	TGraph *graph_rapidity_c = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_c);
	TGraph *graph_rapidity_b = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_b);
	//TGraph *graph_rapidity_t = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_t);

	// draw graps on top of each other
	graph_rapidity_d->SetTitle("qqbar->(Z0+gamma)->mu-mu+ rapidity integrals");
	graph_rapidity_d->GetXaxis()->SetTitle("Sqrt(s_hat) [GeV]");
	graph_rapidity_d->GetYaxis()->SetTitle("rapidity integral");
	graph_rapidity_d->SetLineColor(kBlue);
	graph_rapidity_d->Draw("AC");
	graph_rapidity_u->SetLineColor(kRed);
	graph_rapidity_u->Draw("C");
	graph_rapidity_s->SetLineColor(kBlack);
	graph_rapidity_s->Draw("C");
	graph_rapidity_c->SetLineColor(kGreen);
	graph_rapidity_c->Draw("C");
	graph_rapidity_b->SetLineColor(kYellow);
	graph_rapidity_b->Draw("C");
	//graph_rapidity_t->SetLineColor(kMagenta);
	//graph_rapidity_t->Draw("C");

	TLegend *leg3 = new TLegend(0.91, 0.70, 0.99, 0.99);
	leg3->SetFillColor(20);
	leg3->AddEntry(graph_rapidity_d,"ddbar","L");
	leg3->AddEntry(graph_rapidity_u,"uubar","L");
	leg3->AddEntry(graph_rapidity_s,"ssbar","L");
	leg3->AddEntry(graph_rapidity_c,"ccbar","L");
	leg3->AddEntry(graph_rapidity_b,"bbbar","L");
	leg3->AddEntry(graph_rapidity_b,"ttbar","L");
	leg3->Draw();
	
	string epsfile = commonOps.checkANDsetFilepath("MOSESDATA","/CrossSection_Rapidity.eps");
	c1->Print(graph1.c_str(),"root"); 
	c1->Update();
	c1->Print(epsfile.c_str());
	graphFile1->Write();


	//----------------------

	TCanvas *c2 = new TCanvas("c2","qqbar->(Z0+gamma)->mu-mu+ Z0width",0,0,1400,800);
	c2->SetFillColor(37);
	c2->SetLogy();
	c2->SetGrid();
	TGraph *Z0_width = new TGraph (nentries, Sqrt_s_hat, Z0width);
	Z0_width->SetTitle("qqbar->(Z0+gamma)->mu-mu+ Z0width");
	Z0_width->GetXaxis()->SetTitle("Sqrt(s_hat) [GeV]");
	Z0_width->GetYaxis()->SetTitle("Z0 width [GeV]");
	Z0_width->Draw("AC");
	c2->Print(graph2.c_str(),"root"); 
	c2->Update();
	graphFile2->Write();

	//----------------------


	TCanvas *c3 = new TCanvas("c3","qqbar->(Z0+gamma)->mu-mu+ uubar rapidity integral",0,0,1400,800);
        c3->SetFillColor(37);
        c3->SetLogy();
        c3->SetGrid();
	TGraph *graph_rapidity_largeN = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_largeN);
        TGraph *graph_rapidity_mediumN = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_mediumN);
        TGraph *graph_rapidity_smallN = new TGraph (nentries, Sqrt_s_hat, rapidityIntegral_smallN);

        // draw graps on top of each other
        graph_rapidity_largeN->SetTitle("qqbar->(Z0+gamma)->mu-mu+ uubar rapidity integral for 3 segmentations");
        graph_rapidity_largeN->GetXaxis()->SetTitle("Sqrt(s_hat) [GeV]");
        graph_rapidity_largeN->GetYaxis()->SetTitle("uubar rapidity integral");
        graph_rapidity_largeN->SetLineColor(kRed);
	graph_rapidity_largeN->SetLineWidth(10);
        graph_rapidity_largeN->Draw("AC");
        graph_rapidity_mediumN->SetLineColor(kGreen);
	graph_rapidity_mediumN->SetLineWidth(5);
        graph_rapidity_mediumN->Draw("C");
        graph_rapidity_smallN->SetLineColor(kBlue);
        graph_rapidity_smallN->Draw("C");

        TLegend *leg4 = new TLegend(0.91, 0.70, 0.99, 0.99);
        leg4->SetFillColor(20);
        leg4->AddEntry(graph_rapidity_largeN,"largeN","L");
        leg4->AddEntry(graph_rapidity_mediumN,"mediumN","L");
        leg4->AddEntry(graph_rapidity_smallN,"smallN","L");
        leg4->Draw();
	string epsfile3 = commonOps.checkANDsetFilepath("MOSESDATA","/rapidity.eps");
        c3->Print(graph3.c_str(),"root");
        c3->Update();
        c3->Print(epsfile3.c_str());
        graphFile3->Write();


	cout << "\n" << "done..." << endl;
}

