/**************************************/
/*  class of user defined parameters  */
/**************************************/
#include "parameters.h"
using namespace std;

namespace Moses1 {

parameters::parameters(){}
parameters::~parameters(){}

void parameters::setParameters(physicalParameters &physPrm){ // initialize physical parameters struct (overload)
	m_physPrm.sinThetaW2 = /*0.2238*/ 0.2312015;
	m_physPrm.cosThetaW2 = /*1.-m_physPrm.sinThetaW2*/ 0.7687985;
	m_physPrm.pi = 3.14159265;
	m_physPrm.alpha = 0.00729735256824;
	m_physPrm.gQCD  = 0.11762; // 1!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        m_physPrm.g_mGluonStar  = 0.8; // 1!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	m_physPrm.Gmu = 0.0000116633980690699; // (GeV)^-2
	m_physPrm.mGluonStar = 3000; // Ge
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	m_physPrm.mStar = 4000; // GeV
	m_physPrm.mZprime = 4000; // GeV
	m_physPrm.wZprime = 120; // GeV
//	m_physPrm.mStar = 3000; // GeV
//	m_physPrm.mZprime = 3000; // GeV
//	m_physPrm.wZprime = 80; // GeV
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	m_physPrm.i = dcomplex(0,1);
	m_physPrm.GeV2mbarn = 0.38937930419; // 1/GeV^2 to mb (mili-barn) conversion constant
	m_physPrm.GeV2nbarn = 1e6*0.38937930419; // 1/GeV^2 to nb (nano-barn) conversion constant
	/*-----------------------------------------------------------------------------------------*/
	/*                                                   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
	/* [1] == (hbar*c)^2 = 0.38937930419 [GeV^2 * mb] -> !!! [1/GeV^2] = 0.38937930419[mb] !!! */
	/*                                                   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
	/* sigma ~ [mb] ~ [1/GeV^2]                                                                */ 
	/* d(sigms)/ds ~ [mb/GeV^2] ~ [1/GeV^4]                                                    */
	/* d(sigma)/d(sqrt(s) == d(sigms)/ds * 2sqrt(s) ~ [mb/GeV] ~ [1/GeV^3]                     */
	/* 1[mb] = 1e6[nb]                                                                         */
	/*-----------------------------------------------------------------------------------------*/
	
	physPrm = m_physPrm;
}

void parameters::setParameters(runParameters &runPrm){ // initialize run parameters struct (overload)
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//	m_runPrm.sMax = 14.e3*14.e3 /*14e3 * 14e3*/; // (x*(10^3)GeV)^2 = (xTeV)^2
	m_runPrm.sMax = 7.e3*7.e3;
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	m_runPrm.nEvents = 0;
	m_runPrm.luminosity = 0;
	m_runPrm.etaCut = 2.5; // 1e5
	m_runPrm.pTCut = 10.;  // 0.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	
//	m_runPrm.invMassCut = 1000.;
//	m_runPrm.invMassCut = 900.;
	m_runPrm.invMassCut = 200.;
//	m_runPrm.invMassCut = 150.;
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	m_runPrm.pTsystemCut = 14e3;
	m_runPrm.ySystemCut = -100.;
	//---------------//
	runPrm = m_runPrm;
}

void parameters::printUnits(void) {
	cout << "\n";
	cout << "+---------------------------------- conversion of units ----------------------------------+" << endl;
	cout << "|                                                   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! |" << endl;
	cout << "| [1] == (hbar*c)^2 = 0.38937930419 [GeV^2 * mb] -> !!! [1/GeV^2] = 0.38937930419[mb] !!! |" << endl;
	cout << "|                                                   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! |" << endl;
	cout << "| sigma ~ [mb] ~ [1/GeV^2]                                                                |" << endl;
	cout << "| d(sigms)/ds ~ [mb/GeV^2] ~ [1/GeV^4]                                                    |" << endl;
	cout << "| d(sigma)/d(sqrt(s)) == d(sigms)/ds * 2sqrt(s) ~ [mb/GeV] ~ [1/GeV^3]                    |" << endl;
	cout << "| 1[mb] = 1e6[nb]                                                                         |" << endl;
	cout << "+-----------------------------------------------------------------------------------------+" << endl;
	cout << "\n";
}

}
