/****************************************************************************************/
/* an example program to demonstrate (long description) the building of a cross section */
/****************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Moses1;

int main() {

	cout << "Holy Moses..." << endl;

	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();

	cout << "\n" << "------------------------Set user defined parameters and print them------------------------" << endl;
	
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct

	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	
	cout << "\n\n User defined PHYSICAL parameters:" << endl;
	cout << "sinThetaW2 = " << physPrm.sinThetaW2 << endl;
	cout << "pi = " << physPrm.pi << endl;
	cout << "alpha = " << physPrm.alpha << endl;
	cout << "Gmu = " << physPrm.Gmu << endl;
	cout << "mStar = " << physPrm.mStar << endl;
	cout << "i(complex) = " << physPrm.i << endl;
	cout << "1/GeV^2 to mbarn conversion constant = " << physPrm.GeV2mbarn << endl;
	cout << "---------------------------------" << endl;

	cout << "\n\n User defined RUN parameters:" << endl;
	cout << "sMax = " << runPrm.sMax << endl;
	cout << "nEvents = " << runPrm.nEvents << endl;
	cout << "luminosity = " << runPrm.luminosity << endl;
	cout << "---------------------------------" << endl;

	cout << "set parameters...OK" << endl;
	




	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;
	
	PDTparticle prt;
	prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "PDTparticle class was successfuly initialized" << endl;
	cout << "*******************************************************" << endl;
	cout << "*** NOTE THAT YOU CAN CALL A SPECIFIC PARTICLE      ***" << endl;
	cout << "*** USING PDT ENUM DEFINED IN: HEPPDTparticle.h ***" << endl;
	cout << "*******************************************************" << endl;
	// the constructor will set the table of particle properties,
	// it will initialize the "particle" class and will read from the ***supplied*** PDT table file (particle.tbl)
	// PDT key to fundamental particles:
	// particle: d u s c b t b' t' e- nu_e mu nu_mu tau- nu_tau
	// id:       1 2 3 4 5 6 7  8  11 12   13 14    15   16
	// particle: g  gamma Z0 W+ Higgs0 Z'
	// id:       21 22    23 24 25     32
	/*
		enum pdtEnum { // *** note that this is not a complete list of PDT's particles ***
			PDTDWN=1, PDTUP=2, PDTSTR=3, PDTCHM=4, PDTBOT=5, PDTTOP=6,
			PDTBOTPRIME=7, PDTTOPPRIME=8,
			PDTE=11, PDTNUE=12, PDTMU=13, PDTNUMU=14, PDTTAU=15, PDTNUTAU=16,
			PDTTAUPRIME=17, PDTNUTAUPRIME=18,
			PDTGLU=21, PDTGAMMA=22, PDTZ=23, PDTWPLUS=24
		};
	*/




	cout << "\n\n" << "--------------Set all involved particle properties into 2 vectors--------------" << endl;

	bool print = true;
	
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	cout << "\n" << "in vector1: 12 quarks and leptons" << "\n";
	// fill all quarks and leptons properties in a single vector
	prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS}; // glu, gamma, Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	cout << "\n" << "in vector2: 4 PDTbosons" << "\n";
	prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector
	cout << "set particle properties to 2 vectors...OK" << endl;



	cout << "\n\n" << "--------------Set some more properties manually into the 2 vectors--------------" << endl;
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// now, add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3 2) Nc (Number of possible colors)
	// see: CUSTOMparticle.cxx for more details of how this is actually being done
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	cout << "set extra properties (weakI3, Nc) to the quarks and leptons vector...OK" << endl;
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here.




	
	cout << "\n\n" << "--------------call couplings test--------------" << endl;
	couplings cplng; // call the class.
	// get the photon-e+e-(hlicity=-1/2) couplings
	dcomplex g_gamma_electron = cplng.getCouplings(-0.5, PDTbosons[1], CUSTOMfermions[6], physPrm);
	cout << "photon-electron(hlicity = -1/2) couplings: " << g_gamma_electron << endl;

	// get the Z0-e+e-(hlicity=-1/2) couplings
	dcomplex g_Z0_electron_hplus = cplng.getCouplings(-0.5, PDTbosons[2], CUSTOMfermions[6], physPrm);
	cout << "Z0-electron(hlicity = -1/2) couplings: " << g_Z0_electron_hplus << endl;

	// get the Z0-e+e-(hlicity=+1/2) couplings
	dcomplex g_Z0_electron_hminus = cplng.getCouplings(0.5, PDTbosons[2], CUSTOMfermions[6], physPrm);
	cout << "Z0-electron(hlicity = +1/2) couplings: " << g_Z0_electron_hminus << endl;



	

	cout << "\n\n" << "--------------call width test--------------" << endl;
	partialWidth wdt; // call the class
	// get the partial width of Z0 dacy to e+e-
	double width = wdt.getpartialWidth(g_Z0_electron_hplus, g_Z0_electron_hminus, PDTbosons[2], CUSTOMfermions[6], physPrm);
	cout << "Z0 -> e+e- partial width: " << width  << "\n" << endl;
	
	// calculate the total width of Z0:
	// run over all possible decays of the Z0 bosons to 2 fermions:
	dcomplex gplus, gminus;
	double totalwidth = 0;

	for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
		// calculate the couplings of f fbar to Z0
		// and then the calculate the partial width to this pair of f fbar
		// where f = fermion = CUSTOMfermions[i] = i'th quark/lepton in the vector we defined earlier.

		if (CUSTOMfermions[i].pdt_prt.id != 6) { // Z0 can not decay to top+topbar (m_Z0 < m_top)
			gplus = cplng.getCouplings(+0.5, PDTbosons[2], CUSTOMfermions[i], physPrm);
			gminus = cplng.getCouplings(-0.5, PDTbosons[2], CUSTOMfermions[i], physPrm);
			cout << "g+(" << CUSTOMfermions[i].pdt_prt.name << ") = " << gplus <<
				"\t\t\t g-(" << CUSTOMfermions[i].pdt_prt.name << ") = " << gminus << endl;
			totalwidth += wdt.getpartialWidth(gplus, gminus, PDTbosons[2], CUSTOMfermions[i], physPrm);
		}
	}
	
	cout << "-------------------" << endl;
	cout << "Z0 -> ffbar TOTAL CALCULATED width: " << totalwidth << " (where f = d,u,s,c,b,e,nu_e,mu,nu_mu,tau,nu_tau)" << endl;
	cout << "Z0 PDT FULL width: " << PDTbosons[2].width << endl;
	cout << "The difference = " << 100*abs(PDTbosons[2].width-totalwidth)/PDTbosons[2].width << "%"
		 << " is due to other decay modes which we didnt calculate here..." << endl;






	cout << "\n\n" << "--------------call rapidity simple functions test--------------" << endl;
	rapidity rap;
	double y=0.1;
	double m = 91.2;
	double scale=m*m;
	double xBj_q = rap.getBjorkenx(scale, y, runPrm, false); // get x_q from rapidity and s_max, s_hat
	double xBj_qbar = rap.getBjorkenx(scale, y, runPrm, true); // get x_qbar from rapidity and s_max, s_hat
	cout << "getting x1 & x2 from y=" << y << ":\t x1(q)=" << xBj_q << "\t x2(qbar)=" << xBj_qbar << endl;
	y = rap.getrapidity(xBj_q,xBj_qbar); // get back the rapidity from x1 x2
	cout << "getting back the rapidity(from x1 & x2): y=" << y << endl;
	

	


	cout << "\n\n" << "--------------call rapidity with lha functions test--------------" << endl;
	lhapdf lha; // set the class for LHAPDF call
	
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS","MRST2006nnlo.LHgrid",subset); // can also initialize with full path and name

	int n_ysegments = 1000; // for the integration
	double s_hat_Z0=91.2; // s hat at Z0 mass
	y=0.1; // rapidity

	double rapIntegral=0;
	for (int i=0 ; i<6 ; i++) { // loop over all quarks
		// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
		// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
		//     and here only 3 are valid sice the propogator is colorless
		// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams
		rapIntegral += 2*rap.simpsonIntegral(n_ysegments,s_hat_Z0,CUSTOMfermions[i],runPrm,lha)/CUSTOMfermions[i].Nc;
	}
	cout << "rapidity integral = " << rapIntegral << endl;
			
	





	cout << "\n\n" << "--------------call helicity amplitude test--------------" << endl;
	helicityAmplitude h;
	double sZ0 = 91;
	dcomplex hAmp (0,0);
	customParticle inFermion = CUSTOMfermions[2]; // u quark
	customParticle outFermion = CUSTOMfermions[7]; // muon
	hAmp = h.gethelicityAmplitude(sZ0, totalwidth, -0.5, -0.5, inFermion, outFermion, PDTbosons[1], physPrm); // photon propagator h(--)
	hAmp += h.gethelicityAmplitude(sZ0, totalwidth, -0.5, -0.5, inFermion, outFermion, PDTbosons[2], physPrm); // Z0 propagator h(--)
	cout << "Helicity amplitude(--) for u,ubar->Z0,photon->mu,mubar is: " << hAmp << "\n\n";

	

	// first step towards full cross section function:
	dcomplex fullAmp (0,0);
	double squareAmp = 0;
	double differentialSquareAmp = 0;
	
	Int_t nentries = 1000;
	double s_hat_max = 200*200;
	double s_hat_min = 1;
	double ds_hat = (s_hat_max-s_hat_min)/nentries;
	double s_hat = s_hat_min;
	double qqbarRapidity_shat=0;
	
	// alocate arrays for the graphs
	Double_t *Sqrt_s_hat = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp = (Double_t *)malloc(nentries*sizeof(Double_t));
	cout << "Allocate arrays for the graphs...OK" << endl;

	cout << "\n\nRun over s_hat ([" << sqrt(s_hat_min) << "->" << sqrt(s_hat_max) << "]GeV)^2 and sum over:" << endl;
	cout << "(1) helicity states and (2) all initial quarks pairs" << endl;

	for (Int_t n=0 ; n<nentries ; n++) { // loop over s_hat

		squareAmp = 0; // before each new s_hat it needs to be initialized to '0' !!!!!!!!!!!!!
		
		for (int q=0 ; q<=5 ; q++) { // run over all quarks pairs
			
			differentialSquareAmp = 0; // before each new q it needs to be initialized to '0'
			
			for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states
				for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states
					
					fullAmp = dcomplex(0,0); // before each helicity state it needs to be initialized to '(0,0)'
					
					fullAmp += h.gethelicityAmplitude(s_hat, totalwidth, hIn, hOut,
							CUSTOMfermions[q], outFermion,
							PDTbosons[1], physPrm); // photon propagator
					fullAmp += h.gethelicityAmplitude(s_hat, totalwidth, hIn, hOut,
							CUSTOMfermions[q], outFermion,
							PDTbosons[2], physPrm); // Z0 propagator
					differentialSquareAmp += abs(fullAmp)*abs(fullAmp);
				} // end hOut
			} // end hIn
			
			qqbarRapidity_shat = rap.simpsonIntegral(n_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha); // get the qqbar rapidity integral factor
			// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
			// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
			//     and here only 3 are valid sice the propogator is colorless
			// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams
			differentialSquareAmp *= 2*qqbarRapidity_shat/CUSTOMfermions[q].Nc;

			differentialSquareAmp *= CUSTOMfermions[q].Nc;  // multyply by number of colors for each flavor of incoming fermions

			squareAmp += differentialSquareAmp; // only then add the differential square amplitude to the full one

		} // end q

		squareAmp *= physPrm.pi*physPrm.alpha*physPrm.alpha*s_hat/3;  // multyply by some constants *** and by s_hat *** 
		squareAmp *= 2*sqrt(s_hat); // s_hat -> sqrt(s_hat) Jacobian of the transformation

		// fill the arrays of the graphs:
		Sqare_amp[n] = squareAmp;
		Sqrt_s_hat[n] = sqrt(s_hat);

		cout << "sqrt(s_hat)=" << sqrt(s_hat) << "\r" << flush;
		s_hat += ds_hat;
		
	
	} // end s_hat
	cout << "Created 2 Double_t arrays:"<< endl;
	cout << "(1) Ecm = sqrt(s_hat) [GeV], and (2) Square amplitude for the process: q,qbar -> Z0,photon -> mu,mubar" << endl;





	cout << "\n\n" << "--------------creating graphs--------------" << endl;
	// create the graph and register it to a root file
	commonOperations commonOps;
	string graph = commonOps.checkANDsetFilepath("MOSESDATA","/squareAMPgraph.root");

	TFile *graphFile = new TFile(graph.c_str(),"RECREATE"); // new root file for the graph
	TCanvas *canv = new TCanvas("canv","qqbar->(Z0,gamma)->mu-mu+",0,0,1400,800);
	canv->SetFillColor(37);
	canv->SetLogy();
	canv->SetGrid();

	TGraph *graph_Z0 = new TGraph (nentries, Sqrt_s_hat, Sqare_amp);
	
	// draw graps on top of each other
	graph_Z0->SetTitle("qqbar->(Z0,gamma)->mu-mu+ Square Amplitude");
	graph_Z0->GetXaxis()->SetTitle("Sqrt(s) [GeV]");
	graph_Z0->GetYaxis()->SetTitle("Square Amplitude");
	graph_Z0->SetLineColor(kRed);
	graph_Z0->Draw("AC");
	canv->Print(graph.c_str(),"root"); 
	canv->Update();
	graphFile->Write();



	cout << "\n" << "done..." << endl;
}

